! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ANGADD(DALPH0,DDELT0,DANG,DPOS,DALPHA,DDELTA,IERR)
C
CC  Calculates a new position out of the separation and position angle
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:48
C
CA  author : KOD        date: 07-FEB-1991 19:22
C
CT  status: tested
C
C   general description
CG
CG  This subroutine calculates the coordinates of a position which
CG  is supplied by a reference position and an offset. The offset is
CG  specified by the angular separation and the position angle. The
CG  position angle is defined as the angle between the great circle
CG  passing through both positions and the local north meridian as
CG  seen from the reference position (DALPH0, DDELT0). It is counted
CG  counterclockwise from North over East, South, West to North.
C
C   call_var.          type I/O description
CP
CP  DALPH0              R8  I   Right Ascension of reference  (radians)
CP  DDELT0              R8  I   Declination     of reference  (radians)
CP
CP  DANG                R8  I   angular separation            (radians)
CP  DPOS                R8  I   position angle                (radians)
CP
CP  DALPHA              R8    O resulting Right Ascension     (radians)
CP  DDELTA              R8    O resulting Declination         (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DALPH0, DDELT0, DALPHA, DDELTA, DANG, DPOS,
     &              D1, D2, D3, DPH1, DTH1, DPH2, DTH2, DRADRS
      REAL*4        HFLAG
      INTEGER*4     IERR, IOS, ISTAT
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'ANGADD'/
C
      IERR = 0
C
      if (max(abs(DALPH0),abs(DALPHA)).gt.D2PI) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Unusual value for the R.A.')
      endif
C
      if (DPOS.lt.-DPI .or. DPOS.gt.D2PI) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Unusual value for DPOS')
      endif
C
      if (DANG.lt.0.D0) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'ANG < 0')
      endif
C
C
      if (IERR.lt.2) then
C
        D1 =  0.D0
        D2 =  DRADRS ( DDELT0 - DPID2 )
        D3 =  DRADRS ( -DPID2 - DALPH0 )
C
        DPH1 = DRADRS ( DPID2 - DPOS )
        DTH1 = DANG
C
        call EULER1(D1,D2,D3,DPH1,DTH1,DPH2,DTH2,ISTAT)
C
        if (ISTAT.gt.0) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
C
        else
C
          DALPHA = DPH2
          DDELTA = DPID2 - DTH2
C
        endif
C
      endif
C
C
      RETURN
      END
