! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ANGSEP(DLONG1,DLAT1,DLONG2,DLAT2,DANG,IERR)
C
CC    Calculates the angular separation between two celestial bodies
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:48
C
CA  author : KOD        date: 06-SEP-1989 16:40
CU  update : KOD        date: 20-MAR-1990 15:45
CU  update : KOD        date: 24-OCT-1991 13:57  (sin --> dsin, ...)
CU  update : KOD        date: 17-AUG-1992 09:58  (`1.D-6')
CU  update : KOD        date: 22-JUL-2013 10:35  (`DR = dmin1(DR,1.D0)')
C
CT  status: tested
C
C   general description
CG
CG  Subroutine to calculate the angular separation between two
CG  celestial bodies. The 'latitude' is defined as to have
CG  0 degrees at the equator and +90 degrees at the north pole.
CG  Special care is taken for small angles, according to
CG    R.W.Sinnott, 'Sky and Telescope', August 1984, p.159.
C
C   call_var.          type I/O description
CP
CP  DLONG1              R8  I   'longitude 1' (e.g. R.A.; radians)
CP  DLAT1               R8  I   'latitude  1' (e.g. Dec.; radians)
CP
CP  DLONG2              R8  I   'longitude 2' (e.g. R.A.; radians)
CP  DLAT2               R8  I   'latitude  2' (e.g. Dec.; radians)
CP
CP  DANG                R8    O angular separation       (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$INCLUDE:ICGENL.INC        general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DLONG1, DLAT1, DLONG2, DLAT2, DANG, D1, D2, DR
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'ANGSEP'/
C
      IERR = 0
C
      if (max(abs(DLONG1),abs(DLONG2)).gt.D2PI+1.D-6) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Unusual value for the longitude')
      endif

      if (max(abs(DLAT1),abs(DLAT2)).gt.DPID2+1.D-6) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input parameters')
      endif
C
      if (IERR.lt.2) then
C
        D1 = dsin ( 0.5D0 * ( DLAT1 -  DLAT2 ) )
        D2 = dsin ( 0.5D0 * (DLONG1 - DLONG2 ) )
        DR = D1 * D1 + D2 * D2 * dcos(DLAT1) * dcos(DLAT2)
        DR = dmin1(DR,1.D0)
        DANG = 2.D0 * datan2 ( dsqrt(DR), dsqrt(1.D0-DR) )
C
      endif
C
      RETURN
      END
