! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine CALJD3(chdat,LCHECK,DJD,IERR)
C
CC    Converts the calendar date to the Julian Date
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:43
C
CA  author : KOD        date: 15-NOV-2016
C
C   general description
CG  Subroutine to calculate the Julian Date out of the calendar date.
CG  The calendar date is assumed to be given in the Gregorian system
CG  for dates starting with October 15, 1582, and in the Julian
CG  system until October 4, 1582 (in between it is undefined).
CG
CG  This subroutine is valid for the years  -4712  to  +3500 ,
CG  and has been written according to
CG     van Flandern and Pulkkinen:
CG     'Low-Precision Formulae for Planetary Positions',
CG     Astrophysical Journal Supplement Series 41, 1979, p.392.
CG  and
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.23-25 .
CG
CG  For negative years all the other parameters must be positive.
CG  An extended input check is possible for valid dates.
C
C   call_var.          type I/O description
cp  chdat               c19 i  yyyy-mm-ddThh:mm:ss
CP
CP  LCHECK              L4  I   extended input check if LCHECK=.TRUE.
CP
CP  DJD                 R8    O corresponding Julian Date
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$INCLUDE:ICGENL.INC        general parameter common block
C
C   routines_called    type     description
CR  CNVFRC              SR      conversion to fractional number
CR  JDCAL1              SR      converts Jul. Date into Calendar Date
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
C
      INTEGER*4     IY, IM, ID, IERR, IYR, IMTH, IDAY, IHR, IMT, ISTAT,
     &              JD, IYY, IMM, IHR1, IMT1, IDAT, IOS, isc
      REAL*4        HFLAG, SEC, SEC1
      REAL*8        DJD, DFRC, DH
      LOGICAL*4     LCHECK, LPLUS
      character*(*) chdat
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'CALJD1'/
C
      IERR = 0
c
      read(chdat,'(i4,5(1x,i2))') iy,im,id,ihr,imt,isc
      sec = real(isc)
C
CR!   if wrong input
C
      if ( IY .lt.-4712 .or. IY .gt.3500 .or.
     &     IM .lt.1     .or. IM .gt.12   .or.
     &     ID .lt.1     .or. ID .gt.31   .or.
     &     IHR.lt.0     .or. IHR.gt.24   .or.
     &    (IHR.eq.24 .and. (IMT.gt.0 .or. SEC.gt.0.0)) .or.
     &     IMT.lt.0     .or. IMT.gt.59   .or.
     &     SEC.lt.0.0   .or. SEC.gt.60.0 )   then
C
        IERR = 2
        WRITE(OUTPT,9)  IY,IM,ID
9       FORMAT('Error in input parameters IY,IM,ID ',3(I6,1X))
        call WRERR(RTNAME,IERR,OUTPT)
C
      endif
C
C
      if  (IERR.lt.2)  then
C
C       calculate the Julian Day for 0:00 UT
C
        IDAT = isign((iabs(IY)*100+IM)*100+ID,IY)
C
        if (IDAT.ge.15821015) then
C
C         assume Gregorian calendar date as input:
C
          JD = 367*IY - 7*(IY+(IM+9)/12)/4 - 3*((IY+(IM-9)/7)/100+1)/4
     &       + 275*IM/9 + ID + 1721029
C
        else if (IDAT.le.15821004) then
C
C         assume Julian calendar date as input:
C
          if (IM.gt.2) then
            IYY = IY
            IMM = IM
          else
            IYY = IY - 1
            IMM = IM + 12
          endif
C
          if (IYY.gt.0) then
            DH=0.D0
          else
            DH=0.75D0
          endif
C
          JD = idint(365.25d0*dble(IYY)-DH) +
     &         idint(30.6001d0*dble(IMM+1)) +
     &         ID + 1720995
C
          else
C
            IERR = max(IERR,2)
            WRITE(OUTPT,10)  IDAT
10          FORMAT('Error in date: calendar reform ',I10)
            call WRERR(RTNAME,IERR,OUTPT)
C
        endif
C
C       calculate the fractional part:
C
        LPLUS = .true.
        IHR1 = IHR
        IMT1 = IMT
        SEC1 = SEC
        call CNVFRC(LPLUS,IHR1,IMT1,SEC1,DFRC,ISTAT)
C
CN!     ERROR handling
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        endif
C
C       add both parts
C
        DJD = dble(JD) + DFRC/24.D0 - 0.5D0
C
        IF (LCHECK) THEN
C
CJ!       extended input check
C
          DH = dble(JD) - 0.5D0*0.0
          call JDCAL1(DH,IYR,IMTH,IDAY,IHR1,IMT1,SEC1,ISTAT)
C
CN!       ERROR handling
C
          if  (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,20)
          endif
C
CR!       if inconsistent results
C
          if (IYR .ne.IY  .or. IMTH.ne.IM .or. IDAY.ne.ID) then
            IERR = max(IERR,2)
            WRITE(OUTPT,11) IYR,IY,IMTH,IM,IDAY,ID
11          FORMAT('Error: Date does not exist ',6(I8,1X))
            call WRERR(RTNAME,IERR,OUTPT)
          endif
        endif
      endif
C
CH!   CJD
C
      RETURN
      END
