! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine CMOON1(DJD,DLECL,DBECL,DPRLX,IERR)
C
CC    Calculates the ecl. coordinates and the parallax of the moon
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:54
C
CA  author : KOD        date: 31-AUG-1989 16:14
CU  update : KOD        date: 20-MAR-1990 14:39
C
CT  status: tested (01-SEP-1989 17:18 KOD)
C
C   general description
CG  Subroutine to calculate the ecliptical coordinates and the
CG  parallax of the moon, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.147-153 .
CG
CGT    Accuracy:  10 arcsec in the ecliptical longitude,
CGT                3 arcsec in the ecliptical latitude,
CGT              0.2 arcsec in the parallax.
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Date ( Ephemeris time !)
CP
CP  DLECL               R8    O ecliptical longitude (radians, 0..2*pi)
CP  DBECL               R8    O ecliptical  latitude (radians,-0.5..0.5)
CP  DPRLX               R8    O parallax             (radians, < 0.02 )
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  DDEGRS              R8      'resets' degrees
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DJD, DJD0, DT, DYEAR, DDEGRS, DLECL, DBECL,
     &              DPRLX, DLL, DM, DMM, DD, DF, DOM, DH, DF2, DF3,
     &              DD2, DD3, DD4, DM2, DMM2, DMM3, DMM4, DE1, DE2, DL,
     &              DB, DP, DDLL(4), DDM(4), DDMM(4), DDD(4), DDF(4),
     &              DDOM(4)
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS
      CHARACTER*8   RTNAME
C
      PARAMETER ( DJD0 = 2415020.D0 )
      PARAMETER ( DYEAR =  36525.D0 )
C
      DATA          RTNAME /'CMOON1'/
C
      DATA DDLL / 270.434164D0, 481267.8831D0,-0.001133D0, 0.0000019D0 /
      DATA DDM  / 358.475833D0,  35999.0498D0,-0.000150D0,-0.0000033D0 /
      DATA DDMM / 296.104608D0, 477198.8491D0, 0.009192D0, 0.0000144D0 /
      DATA DDD  / 350.737486D0, 445267.1142D0,-0.001436D0, 0.0000019D0 /
      DATA DDF  /  11.250889D0, 483202.0251D0,-0.003211D0,-0.0000003D0 /
      DATA DDOM / 259.183275D0,  -1934.1420D0, 0.002078D0, 0.0000022D0 /
C
      IERR = 0
C
      if (DJD.lt.0.D0 .or. DJD.gt.3.D6) then
        IERR = 2
        call WRERR(RTNAME,IERR,'Error in the input parameter DJD')
      endif
C
      if (IERR.lt.2) then
C
        DT = (DJD - DJD0) / DYEAR
C
        DLL = DDLL(1) + (DDLL(2) + (DDLL(3) + DDLL(4) * DT) * DT ) * DT
        DM  = DDM (1) + (DDM (2) + (DDM (3) + DDM (4) * DT) * DT ) * DT
        DMM = DDMM(1) + (DDMM(2) + (DDMM(3) + DDMM(4) * DT) * DT ) * DT
        DD  = DDD (1) + (DDD (2) + (DDD (3) + DDD (4) * DT) * DT ) * DT
        DF  = DDF (1) + (DDF (2) + (DDF (3) + DDF (4) * DT) * DT ) * DT
        DOM = DDOM(1) + (DDOM(2) + (DDOM(3) + DDOM(4) * DT) * DT ) * DT
C
        DLL = DDEGRS(DLL)
        DM  = DDEGRS(DM )
        DMM = DDEGRS(DMM)
        DD  = DDEGRS(DD )
        DF  = DDEGRS(DF )
        DOM = DDEGRS(DOM)
C
        DH = sin ( 0.8936086D0 + 0.3525565D0 * DT )
C
        DLL = DLL + 0.000233 * DH
        DM  = DM  - 0.001778 * DH
        DMM = DMM + 0.000817 * DH
        DD  = DD  + 0.002011 * DH
C
        DH = sin (6.048613D0 + ( 2.319019D0 - 1.601008D-4 * DT ) * DT)
C
        DH = 0.003964D0 * DH
C
        DLL = DLL + DH
        DMM = DMM + DH
        DD  = DD  + DH
        DF  = DF  + DH
C
        DH = sin ( DOM * DPI180 )
C
        DLL = DLL + 0.001964D0 * DH
        DMM = DMM + 0.002541D0 * DH
        DD  = DD  + 0.001964D0 * DH
        DF  = DF  - 0.024691D0 * DH
C
        DH = sin ( (DOM + 275.05D0 - 2.30D0) * DPI180 )
C
        DF = DF - 0.004328D0 * DH
C
        DM  = DM  * DPI180
        DMM = DMM * DPI180
        DD  = DD  * DPI180
        DF  = DF  * DPI180
        DOM = DOM * DPI180
C
        DF2  = DF   + DF
        DF3  = DF2  + DF
        DD2  = DD   + DD
        DD3  = DD2  + DD
        DD4  = DD3  + DD
        DM2  = DM   + DM
        DMM2 = DMM  + DMM
        DMM3 = DMM2 + DMM
        DMM4 = DMM3 + DMM
C
        DE1 = 1.D0 - ( 0.002495D0 + 0.00000752D0 * DT ) * DT
        DE2 = DE1 * DE1
C
        DL = DLL
        DL = DL + 6.288750D0 * sin(DMM)
        DL = DL + 1.274018D0 * sin(DD2-DMM)
        DL = DL + 0.658309D0 * sin(DD2)
        DL = DL + 0.213616D0 * sin(DMM2)
        DL = DL - 0.185596D0 * sin(DM)           * DE1
        DL = DL - 0.114336D0 * sin(DF2)
        DL = DL + 0.058793D0 * sin(DD2-DMM2)
        DL = DL + 0.057212D0 * sin(DD2-DM-DMM)   * DE1
        DL = DL + 0.053320D0 * sin(DD2+DMM)
        DL = DL + 0.045874D0 * sin(DD2-DM)       * DE1
        DL = DL + 0.041024D0 * sin(DMM-DM)       * DE1
        DL = DL - 0.034718D0 * sin(DD)
        DL = DL - 0.030465D0 * sin(DM+DMM)       * DE1
        DL = DL + 0.015326D0 * sin(DD2-DF2)
        DL = DL - 0.012528D0 * sin(DF2+DMM)
        DL = DL - 0.010980D0 * sin(DF2-DMM)
        DL = DL + 0.010674D0 * sin(DD4-DMM)
        DL = DL + 0.010034D0 * sin(DMM3)
        DL = DL + 0.008548D0 * sin(DD4-DMM2)
        DL = DL - 0.007910D0 * sin(DM-DMM+DD2)    * DE1
        DL = DL - 0.006783D0 * sin(DD2+DM)        * DE1
        DL = DL + 0.005162D0 * sin(DMM-DD)
        DL = DL + 0.005000D0 * sin(DM+DD)         * DE1
        DL = DL + 0.004049D0 * sin(DMM-DM+DD2)    * DE1
        DL = DL + 0.003996D0 * sin(DMM2+DD2)
        DL = DL + 0.003862D0 * sin(DD4)
        DL = DL + 0.003665D0 * sin(DD2-DMM3)
        DL = DL + 0.002695D0 * sin(DMM2-DM)       * DE1
        DL = DL + 0.002602D0 * sin(DMM-DF2-DD2)
        DL = DL + 0.002396D0 * sin(DD2-DM-DMM2)   * DE1
        DL = DL - 0.002349D0 * sin(DMM+DD)
        DL = DL + 0.002249D0 * sin(DD2-DM2)       * DE2
        DL = DL - 0.002125D0 * sin(DMM2+DM)       * DE1
        DL = DL - 0.002079D0 * sin(DM2)           * DE2
        DL = DL + 0.002059D0 * sin(DD2-DMM-DM2)   * DE2
        DL = DL - 0.001773D0 * sin(DMM+DD2-DF2)
        DL = DL - 0.001595D0 * sin(DF2+DD2)
        DL = DL + 0.001220D0 * sin(DD4-DM-DMM)    * DE1
        DL = DL - 0.001110D0 * sin(DMM2+DF2)
        DL = DL + 0.000892D0 * sin(DMM-DD3)
        DL = DL - 0.000811D0 * sin(DM+DMM+DD2)    * DE1
        DL = DL + 0.000761D0 * sin(DD4-DM-DMM2)   * DE1
        DL = DL + 0.000717D0 * sin(DMM-DM2)       * DE2
        DL = DL + 0.000704D0 * sin(DMM-DM2-DD2)   * DE2
        DL = DL + 0.000693D0 * sin(DM-DMM2+DD2)   * DE1
        DL = DL + 0.000598D0 * sin(DD2-DM-DF2)    * DE1
        DL = DL + 0.000550D0 * sin(DMM+DD4)
        DL = DL + 0.000538D0 * sin(DMM4)
        DL = DL + 0.000521D0 * sin(DD4-DM)        * DE1
        DL = DL + 0.000486D0 * sin(DMM2-DD)
C
        DB =      5.128189D0 * sin(DF)
        DB = DB + 0.280606D0 * sin(DMM+DF)
        DB = DB + 0.277693D0 * sin(DMM-DF)
        DB = DB + 0.173238D0 * sin(DD2-DF)
        DB = DB + 0.055413D0 * sin(DD2+DF-DMM)
        DB = DB + 0.046272D0 * sin(DD2-DF-DMM)
        DB = DB + 0.032573D0 * sin(DD2+DF)
        DB = DB + 0.017198D0 * sin(DMM2+DF)
        DB = DB + 0.009267D0 * sin(DD2+DMM-DF)
        DB = DB + 0.008823D0 * sin(DMM2-DF)
        DB = DB + 0.008247D0 * sin(DD2-DM-DF)     * DE1
        DB = DB + 0.004323D0 * sin(DD2-DF-DMM2)
        DB = DB + 0.004200D0 * sin(DD2+DF+DMM)
        DB = DB + 0.003372D0 * sin(DF-DM-DD2)     * DE1
        DB = DB + 0.002472D0 * sin(DD2+DF-DM-DMM) * DE1
        DB = DB + 0.002222D0 * sin(DD2+DF-DM)     * DE1
        DB = DB + 0.002072D0 * sin(DD2-DF-DM-DMM) * DE1
        DB = DB + 0.001877D0 * sin(DF-DM+DMM)     * DE1
        DB = DB + 0.001828D0 * sin(DD4-DF-DMM)
        DB = DB - 0.001803D0 * sin(DF+DM)         * DE1
        DB = DB - 0.001750D0 * sin(DF3)
        DB = DB + 0.001570D0 * sin(DMM-DM-DF)     * DE1
        DB = DB - 0.001487D0 * sin(DF+DD)
        DB = DB - 0.001481D0 * sin(DF+DM+DMM)     * DE1
        DB = DB + 0.001417D0 * sin(DF-DM-DMM)     * DE1
        DB = DB + 0.001350D0 * sin(DF-DM)         * DE1
        DB = DB + 0.001330D0 * sin(DF-DD)
        DB = DB + 0.001106D0 * sin(DF+DMM3)
        DB = DB + 0.001020D0 * sin(DD4-DF)
        DB = DB + 0.000833D0 * sin(DF+DD4-DMM)
        DB = DB + 0.000781D0 * sin(DMM-DF3)
        DB = DB + 0.000670D0 * sin(DF+DD4-DMM2)
        DB = DB + 0.000606D0 * sin(DD2-DF3)
        DB = DB + 0.000597D0 * sin(DD2+DMM2-DF)
        DB = DB + 0.000492D0 * sin(DD2+DMM-DM-DF) * DE1
        DB = DB + 0.000450D0 * sin(DMM2-DF-DD2)
        DB = DB + 0.000439D0 * sin(DMM3-DF)
        DB = DB + 0.000423D0 * sin(DF+DD2+DMM2)
        DB = DB + 0.000422D0 * sin(DD2-DF-DMM3)
        DB = DB - 0.000367D0 * sin(DM+DF+DD2-DMM) * DE1
        DB = DB - 0.000353D0 * sin(DM+DF+DD2)     * DE1
        DB = DB + 0.000331D0 * sin(DF+DD4)
        DB = DB + 0.000313D0 * sin(DD2+DF-DM+DMM) * DE1
        DB = DB + 0.000306D0 * sin(DD2-DM2-DF)    * DE2
        DB = DB - 0.000283D0 * sin(DMM+DF3)
C
        DH =      0.0004664 * cos(DOM)
        DH = DH + 0.0000754 * cos(DOM + 4.8005281D0 - 0.04014257D0*DT)
C
        DB = DB * (1.D0 - DH)
C
        DP =      0.950724
        DP = DP + 0.051818 * cos(DMM)
        DP = DP + 0.009531 * cos(DD2-DMM)
        DP = DP + 0.007843 * cos(DD2)
        DP = DP + 0.002824 * cos(DMM2)
        DP = DP + 0.000857 * cos(DD2+DMM)
        DP = DP + 0.000533 * cos(DD2-DM)          * DE1
        DP = DP + 0.000401 * cos(DD2-DM-DMM)      * DE1
        DP = DP + 0.000320 * cos(DMM-DM)          * DE1
        DP = DP - 0.000271 * cos(DD)
        DP = DP - 0.000264 * cos(DM+DMM)          * DE1
        DP = DP - 0.000198 * cos(DF2-DMM)
        DP = DP + 0.000173 * cos(DMM3)
        DP = DP + 0.000167 * cos(DD4-DMM)
        DP = DP - 0.000111 * cos(DM)              * DE1
        DP = DP + 0.000103 * cos(DD4-DMM2)
        DP = DP - 0.000084 * cos(DMM2-DD2)
        DP = DP - 0.000083 * cos(DD2+DM)          * DE1
        DP = DP + 0.000079 * cos(DD2+DMM2)
        DP = DP + 0.000072 * cos(DD4)
        DP = DP + 0.000064 * cos(DD2-DM+DMM)      * DE1
        DP = DP - 0.000063 * cos(DD2+DM-DMM)      * DE1
        DP = DP + 0.000041 * cos(DM+DD)           * DE1
        DP = DP + 0.000035 * cos(DMM2-DM)         * DE1
        DP = DP - 0.000033 * cos(DMM3-DD2)
        DP = DP - 0.000030 * cos(DMM+DD)
        DP = DP - 0.000029 * cos(DF2-DD2)
        DP = DP - 0.000029 * cos(DMM2+DM)         * DE1
        DP = DP + 0.000026 * cos(DD2-DM2)         * DE2
        DP = DP - 0.000023 * cos(DF2-DD2+DMM)
        DP = DP + 0.000019 * cos(DD4-DM-DMM)      * DE1
C
        DL    = DL * DPI180
        DLECL = DDEGRS(DL)
C
        DBECL = DB * DPI180
        DPRLX = DP * DPI180
C
      endif
C
      RETURN
      END
