! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine CPREC3(DZETA,DZ,DTHETA,DLOLD,DBOLD,DLNEW,DBNEW,IERR)
C
CC    Converts equatorial coordinates from one equinox to another
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:55
C
CA  author : KOD        date: 08-FEB-1990 14:08
CU  update : KOD        date: 20-MAR-1990 14:42
CU  update : KOD        date: 17-AUG-1992 10:02 (`1.D-6')
C
CT  status: tested
C
C   general description
CG
CG  Subroutine to convert the equatorial coordinates from one
CG  equinox to another, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.65,66.
CG
CGT Notes: The effect of the proper motion must be calculated BEFORE
CGT        converting the coordinates to another epoch.
CGT
CGT        This subroutine should be used for converting many
CGT        coordinates at the same time to avoid unneccessary
CGT        re-calculations of the quantities required for the
CGT        equinox conversion.
C
C   call_var.          type I/O description
CP  DZETA               R8  I   'zeta'   (radians)
CP  DZ                  R8  I   'z'      (radians)
CP  DTHETA              R8  I   'theta'  (radians)
CP
CP  DLOLD               R8  I   old longitude  (radians)
CP  DBOLD               R8  I   old latitude   (radians)
CP
CP  DLNEW               R8    O new longitude  (radians)
CP  DBNEW               R8    O new latitude   (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$INCLUDE:ICGENL.INC        general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  REPOL1              SR      rectangular to polar coord. conversion
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DZETA, DZ, DTHETA, DLOLD, DBOLD, DLNEW, DBNEW,
     &              DL, DCL, DSL, DCB, DSB, DCT, DST, DA, DB, DC,
     &              DH1, DR, DPHI, DH2, DRADRS
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'CPREC3'/
C
      IERR = 0
C
      if (dmax1(dabs(DZETA),dabs(DZ),dabs(DTHETA)).gt.1.D-2 .or.
     &    DLOLD.lt.-1.D-6 .or. DLOLD.gt.D2PI+1.D-6) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Warning: unusual input values')
      endif
C
      if (dabs(DBOLD).gt.DPID2+1.D-6) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input parameter DBOLD')
      endif
C
      if (IERR.lt.2) then
C
        DL = DLOLD + DZETA
        DCL = cos(DL)
        DSL = sin(DL)
C
        DCB = cos(DBOLD)
        DSB = sin(DBOLD)
C
        DCT = cos(DTHETA)
        DST = sin(DTHETA)
C
        DA = DCB * DSL
        DB = DCT * DCB * DCL - DST * DSB
        DC = DST * DCB * DCL + DCT * DSB
C
        DH1 = 0.D0
        call REPOL1(DB,DA,DH1,DR,DPHI,DH2)
        DLNEW = DRADRS(DPHI + DZ)
C
        DBNEW = asin(DC)
        if (abs(DBNEW).gt.0.5D0*DPID2) DBNEW = sign(acos(DR),DBNEW)
C
      endif
C
C
      RETURN
      END
