! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine CSUN01(DJD,LMEAN,DLONG,DDIST,IERR)
C
CC    Calculates the longitude and distance of the sun for a given JD
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:56
C
CA  author : KOD        date: 07-SEP-1989 10:39
CU  update : KOD        date: 20-MAR-1990 14:43
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the true/apparent longitude of the sun
CG  for a given Julian Date, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.79-80 .
CG
CG  The true longitude is referred to the mean equinox of the date,
CG  required e.g. for calculating the geocentric planetary positions.
CG  The apparent longitude is referred to the true equinox of the
CG  date, corrected for nutation and aberration.
CG
CG  The longitude with respect to the standard equinox of 1950.0 can
CG  be calculated with a sufficient accuracy for the 20th century as
CG
CG    DLONG(1950.0) = DLONG - 2.4365E-4 * (year - 1950)  (radians)
CG
CG  Accuracy: 2 arc seconds
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Date
CP
CP  LMEAN               L4  I   true or app. longitude:
CP                                LMEAN = .true.  : true longitude
CP                                LMEAN = .false. : app. longitude
CP
CP  DLONG               R8    O longitude of the sun    (radians)
CP  DDIST               R8    O distance  of the sun  (astr. units)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  EANOM1              SR      calculates eccentric anomaly
CR  ETANOM              SR      converts eccentric into true anomaly
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DJD, DLONG, DJD0, DYEAR, DT, DLL(3), DMM(4), DEE(3),
     &              DL, DM, DE, DTM, DRADRS, DEM, DH, DOM, DDIST,
     &              DHA, DHB, DHC, DHD, DHE, DHH, DCOR
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LMEAN, LDIR
      CHARACTER*8   RTNAME
C
      PARAMETER ( DJD0 = 2415020.D0 )
      PARAMETER ( DYEAR =  36525.D0 )
C
      DATA          RTNAME /'CSUN01'/
C
      DATA DLL / 279.69668D0, 36000.76892D0,  0.0003025D0 /
      DATA DMM / 358.47583D0, 35999.04975D0, -0.000150D0, -0.0000033D0 /
      DATA DEE / 0.01675104D0, -0.0000418D0, -0.000000126D0 /
C
      IERR = 0
C
      if (DJD.lt.0.D0 .or. DJD.gt.3.D6) then
        IERR = 2
        call WRERR(RTNAME,IERR,'Error in the input parameter DJD')
      endif
C
      if (IERR.lt.2) then
C
        DT = (DJD - DJD0) / DYEAR
C
C       calculate the geocentric mean longitude of the sun:
C
        DL = DLL(1) + ( DLL(2) + DLL(3) * DT ) * DT
        DL = DL * DPI180
        DL = DRADRS(DL)
C
C       calculate the mean anomaly:
C
        DM = DMM(1) + ( DMM(2) + ( DMM(3) + DMM(4) * DT ) * DT ) * DT
        DM = DM * DPI180
        DM = DRADRS(DM)
C
C       calculate the eccentricity of the earth`s orbit:
C
        DE = DEE(1) + ( DEE(2) + DEE(3) * DT ) * DT
C
C       calculate the eccentric anomaly:
C
        call EANOM1(DM,DE,DEM,ISTAT)
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        else
C
C         calculate the true anomaly:
C
          LDIR=.true.
          call ETANOM(DE,LDIR,DEM,DTM,ISTAT)
C
          if (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,20)
          else
            DH = DL + DTM - DM
            if (.not.LMEAN) then
              DOM = 4.52354D0 - 33.75715D0 * DT
              DH = DH - 9.9309D-5 - 8.360D-5 * sin(DOM)
            endif
C
            DHA = 153.23D0 + 22518.7541D0 * DT
            DHB = 216.57D0 + 45037.5082D0 * DT
            DHC = 312.69D0 + 32964.3577D0 * DT
            DHD = 350.74D0 + (445267.1142D0 - 0.00144D0 * DT) * DT
            DHE = 231.19D0 + 20.20D0 * DT
            DHH = 353.40D0 + 65928.7155D0 * DT
C
            DHA = DHA * DPI180
            DHB = DHB * DPI180
            DHC = DHC * DPI180
            DHD = DHD * DPI180
            DHE = DHE * DPI180
            DHH = DHH * DPI180
C
            DCOR = 0.00134D0 * cos(DHA)
     &           + 0.00154D0 * cos(DHB)
     &           + 0.00200D0 * cos(DHC)
     &           + 0.00179D0 * sin(DHD)
     &           + 0.00178D0 * sin(DHE)
C
            DH = DH + DCOR * DPI180
            DLONG = DRADRS(DH)
C
            DH = 1.0000002D0 * (1.D0 - DE * cos(DEM))
C
            DCOR = 0.00000543D0 * sin(DHA)
     &           + 0.00001575D0 * sin(DHB)
     &           + 0.00001627D0 * sin(DHC)
     &           + 0.00003076D0 * cos(DHD)
     &           + 0.00000927D0 * sin(DHH)
C
            DDIST = DH + DCOR
C
          endif
C
        endif
      endif
C
      RETURN
      END
