! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine CSUN02(DJD,DJD0,DX,DY,DZ,IERR)
C
CC    Calculates the rect. coordinates of the sun for a given equinox
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:56
C
CA  author : KOD        date: 26-SEP-1989 18:25
CU  update : KOD        date: 30-MAR-1990 14:46
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the rectangular coordinates of the sun
CG  for a given Julian Date with respect to an arbitrary equinox,
CG  according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.85-87 .
CG
CGT Note: The equinox of 1950.0 corresponds to JD 2433282.423,
CGT       the equinox of 1900.0 corresponds to JD 2415020.313,
CGT       according to J.Meeus, p.65 .
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Date for the sun position
CP  DJD0                R8  I   Julian Date defining the equinox
CP
CP  DX                  R8    O x-coordinate (to vernal equinox)
CP  DY                  R8    O y-coordinate (to ecl.longitude 90 deg)
CP  DZ                  R8    O z-coordinate (to north pole)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
C
C   routines_called    type     description
CR  CVEQUI              SR      calculates quantities for epoch conv.
CR  CSUN01              SR      calculates long. and dist. of the sun
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
CR  JDEPS1              SR      calc. the obliquity of the ecliptic
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
C
      REAL*8        DJD, DJD0, DX, DY, DZ, DZETA, DZT, DTHETA, DLONG,
     &              DDIST, DEPS, DCU, DSU, DCZ, DSZ, DCT, DST, DXX,
     &              DXY, DXZ, DYX, DYY, DYZ, DZX, DZY, DZZ, DX0, DY0,
     &              DZ0, DJ, DJ0
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LMEAN
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'CSUN02'/
C
      IERR = 0
C
C     check input values:
C
      if (min(DJD0,DJD).lt.0.D0 .or. max(DJD0,DJD).gt.3.D6) then
        IERR = 2
        call WRERR(RTNAME,IERR,'Error in the input parameter DJD')
      endif
C
C     calculate parameters for converting the equinox:
C
      if (IERR.lt.2) then
        DJ = DJD
        DJ0 = DJD0
        call CVEQUI(DJ,DJ0,DZETA,DZT,DTHETA,ISTAT)
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        endif
      endif
C
C     calculate position of the sun for the initial equinox:
C
      if (IERR.lt.2) then
        LMEAN = .true.
        call CSUN01(DJ,LMEAN,DLONG,DDIST,ISTAT)
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,20)
        endif
      endif
C
C     calculate obliquity of the ecliptic for the initial equinox:
C
      if (IERR.lt.2) then
        LMEAN = .true.
        call JDEPS1(DJ,LMEAN,DEPS,ISTAT)
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,30)
        endif
      endif
C
      if (IERR.lt.2) then
C
C       calculate rectangular positions for the initial equinox:
C
        DX0 = DDIST * cos(DLONG)
        DY0 = DDIST * sin(DLONG) * cos(DEPS)
        DZ0 = DDIST * sin(DLONG) * sin(DEPS)
C
C       calculate parameters for converting the equinox:
C
        DCU = cos(DZETA)
        DSU = sin(DZETA)
C
        DCZ = cos(DZT)
        DSZ = sin(DZT)
C
        DCT = cos(DTHETA)
        DST = sin(DTHETA)
C
        DXX =  DCU * DCZ * DCT - DSU * DSZ
        DXY =  DSU * DCZ + DCU * DSZ * DCT
        DXZ =  DCU * DST
C
        DYX = -DCU * DSZ - DSU * DCZ * DCT
        DYY =  DCU * DCZ - DSU * DSZ * DCT
        DYZ = -DSU * DST
C
        DZX = -DCZ * DST
        DZY = -DSZ * DST
        DZZ =  DCT
C
C       convert rectangular coordinates to the final equinox:
C
        DX = DXX * DX0 + DYX * DY0 + DZX * DZ0
        DY = DXY * DX0 + DYY * DY0 + DZY * DZ0
        DZ = DXZ * DX0 + DYZ * DY0 + DZZ * DZ0
C
      endif
C
      RETURN
      END
