! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine EANOM1(DMANOM,DECC,DEANOM,IERR)
C
CC    Calculates the eccentric anomaly from the mean anomaly
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:59
C
CA  author : KOD        date: 31-AUG-1989 13:57
CU  update : KOD        date: 20-MAR-1990 14:50
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the eccentric anomaly from the mean
CG  anomaly according to the second method described by
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.94-95 .
CG
CG  This subroutine has been tested with a random number generator
CG  used to produce random mean anomalies between 0.0 and 2*pi,
CG  and random eccentricities:
CG
CGT   DECC = 0.90 .. 0.97   n = 100000  It = 5.8  7.6 CPU sec (IBM)
CGT   DECC = 0.00 .. 0.97   n = 100000  It = 4.7  6.8 CPU sec (IBM)
CG
CG  In every case the algorithm converged to an accurracy of 1.E-12
CG  in the eccentric anomaly; above DECC = 0.97 errors occurred.
C
C   call_var.          type I/O description
CP  DMANOM              R8  I   mean anomaly  (radians, 0 .. 2*pi)
CP  DECC                R8  I   numeric eccentricity (0.0 .. 0.97)
CP
CP  DEANOM              R8    O eccentric anomaly (radians, 0 .. 2*pi)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$INCLUDE:ICGENL.INC        general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DMANOM, DECC, DEANOM, DACC, DE, DH, DRADRS
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, MAXIT, IT
      LOGICAL*4     LCONV
      CHARACTER*8   RTNAME
C
      PARAMETER ( DACC = 1.D-12 )
C
      PARAMETER ( MAXIT = 100 )
C
      DATA          RTNAME /'EANOM1'/
C
      IERR = 0
C
      if (DMANOM.lt.0.d0 .or. DMANOM.gt.D2PI) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Input parameter DMANOM out of range')
      endif
C
      if (DECC.lt.0.D0 .or. DECC.ge.0.97D0) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Input parameter DECC out of range')
      endif
C
      if (IERR.lt.2) then
C
        IT = 0
        LCONV = .false.
        DE = DMANOM
C
        do while (.not.LCONV .and. IT.lt.MAXIT)
          IT = IT + 1
          DH = (DMANOM + DECC*sin(DE) - DE) / (1.D0 - DECC*cos(DE))
          DE = DE + DH
          LCONV = (abs(DH).lt.DACC)
        end do
C
        if (.not.LCONV) then
          IERR = max(IERR,2)
          call WRERR(RTNAME,IERR,'No sufficient convergence')
        else
          DEANOM = DRADRS(DE)
        endif
C
      endif
C
C
      RETURN
      END
