! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ELEVA1(DLAT,DDELTA,DHGHT,LREF,DHR,DAZ,IERR)
C
CC    Calculates the hour angle corresponding to a given elevation
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:59
C
CA  author : KOD        date: 29-SEP-1989 16:36
CU  update : KOD        date: 30-MAR-1990 15:00
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the hour angle (and azimuth) for
CG  a celestial object when its apparent/true elevation DECREASES
CG  below a certain value (equivalent to an hour angle between
CG  0 and +12 hours). The apparent elevation takes the effect
CG  of atmospheric refraction into account.
CG
CG  If the elevation cannot be reached, the hour angle is set
CG  according to the best approximation (0 or pi) and a warning
CG  message will be given ( IERR = 1 ) .
CG
CG  The azimuth is defined to start from south and increase
CG  towards west.
CG
CG  To determine the hour angle for apparent rising (or setting),
CG  take the following values:
CG
CGT    apparent height    true height
CGT    ---------------   -------------
CGT       0' =  0.000    -34' = -0.010    for stars and planets,
CGT     -13' = -0.004    -50' = -0.015    for the sun, and
CGT     +36' =  0.010    + 8' = +0.002    for the moon.
CG
CG  According to Jean Meeus, 'Astronomical Formulae for Calculators',
CG                           third edition, 1985, p.44, 207, 208 .
C
C   call_var.          type I/O description
CP  DLAT                R8  I   geographical latitude     (radians)
CP
CP  DDELTA              R8  I   declination               (radians)
CP
CP  DHGHT               R8  I   height above the horizon  (radians)
CP
CP  LREF                L4  I   correction for atmospheric refraction:
CP                                LREF = .true.  : with correction
CP                                LREF = .false. : without correction
CP
CP  DHR                 R8    O hour angle     ( 0 .. pi , radians)
CP  DAZ                 R8    O azimuth        ( 0 .. pi , radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  REFRA1              SR      corrects for atmospheric refraction
CR  REPOL1              SR      converts rectangular into polar coord.
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DHGHT, DDELTA, DLAT, DHR, DAZ, DH1, DH2, DH,
     &              DR, DTHETA, DHT
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LDIR, LREF
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'ELEVA1'/
C
      IERR = 0
C
      if (max(abs(DLAT),abs(DDELTA),abs(DHGHT)).gt.DPID2) then
C
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input values')
C
      else
C
C       correct for atmospheric refraction
C
        if (.not.LREF) then
          DHT = DHGHT
        else
          LDIR=.true.
          DH = DHGHT
          call REFRA1(DH,LDIR,DHT,ISTAT)
          if  (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,10)
          endif
        endif
C
      endif
C
      if (IERR.lt.2) then
C
        DH1 = sin(DHT) - sin(DLAT) * sin(DDELTA)
        DH2 = cos(DLAT) * cos(DDELTA)
C
        if (DH2.eq.0.D0) then
C
          DHR = 0.D0
          DAZ = 0.D0
C
          if ( .not. ((cos(DLAT)  .eq.0.D0 .and. DDELTA.eq.DHT) .or.
     &                (cos(DDELTA).eq.0.D0 .and. DLAT  .eq.DHT) )) then
            IERR = max(IERR,1)
            call WRERR(RTNAME,IERR,'Elevation cannot be reached')
          endif
C
        else
C
          DH = DH1 / DH2
C
          if (dabs(DH).gt.1.D0) then
            IERR = max(IERR,1)
            call WRERR(RTNAME,IERR,'Elevation cannot be reached')
            DH = sign(1.D0,DH)
          endif
C
          DHR = acos(DH)
C
C         calculate the azimuth:
          DH1 = DH * sin(DLAT) - tan(DDELTA) * cos(DLAT)
          DH2 = sin(DHR)
          DH = 0.D0
          call REPOL1(DH1,DH2,DH,DR,DAZ,DTHETA)
C
        endif
C
      endif
C
      RETURN
      END
