! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ELEVA2(DJD,DLONG,DLAT,DALPHA,DDELTA,DHGHT,LREF,
     &                  DJD1,DAZ1,DJD2,DAZ2,IERR)
C
CC    Calculates the times corresponding to a given elevation
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:59
C
CA  author : KOD        date: 29-SEP-1989 16:56
CU  update : KOD        date: 30-MAR-1990 15:05
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the times (and corresponding azimuths)
CG  during which the apparent/true elevation of a celestial object
CG  EXCEEDS a given value. The apparent elevation takes the effect
CG  of atmospheric refraction into account.
CG
CG  If the elevation cannot be reached the times will be calculated
CG  for the corresponding transit and a warning message will be
CG  given ( IERR = 1 ) .  The azimuth is defined to start from
CG  south and increase towards west.
CG
CG  According to a resolution of the IAU east longitudes are counted
CG  positive, west longitudes negative
CG                                  (Sky and Tel, June, 1984, p.559).
CG
CG  To determine the hour angle for apparent rising (or setting),
CG  take the following values:
CG
CGT   apparent height    true height
CGT   ---------------   -------------
CGT      0' =  0.000    -34' = -0.010   for stars and planets,
CGT    -13' = -0.004    -50' = -0.015   for the sun, and
CGT    -13' = -0.004    -50' = -0.015   for the moon (with par.cor.)
CGT    +36' =  0.010    + 8' = +0.002   for the moon ( no  par.cor.)
CG
CG  The times are given in Julian Ephemeris Times (approx. UT) and
CG  are selected to be as close to the given date as possible.
cg  +++  problem: djd < djd1 < djd2 possible  +++
CG
CG  According to Jean Meeus, 'Astronomical Formulae for Calculators',
CG                           third edition, 1985, p.44, 207, 208 .
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Day to be analysed
CP
CP  DLONG               R8  I   geographical longitude    (radians)
CP  DLAT                R8  I   geographical latitude     (radians)
CP
CP  DALPHA              R8  I   right ascension           (radians)
CP  DDELTA              R8  I   declination               (radians)
CP
CP  DHGHT               R8  I   height above the horizon  (radians)
CP
CP  LREF                L4  I   correction for atmospheric refraction:
CP                                LREF = .true.  : with correction
CP                                LREF = .false. : without correction
CP
CP  DJD1                R8    O J.D. for ascending        ( < DJD2 )
CP  DAZ1                R8    O azimuth for DJD1          (radians)
CP
CP  DJD2                R8    O J.D. for descending       ( > DJD1 )
CP  DAZ2                R8    O azimuth for DJD2          (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  ELEVA1              SR      calculates the hour angle for elevation
CR  LSTJD1              SR      calculates the time from sid. time
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DJD, DLONG, DLAT, DALPHA, DDELTA, DHGHT, DJD1,
     &              DAZ1, DJD2, DAZ2, DHR, DAZ, DLSID, DJD0, DT,
     &              DJ, DLG, DLT, DD, DHG
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LMEAN, LREF, LRF
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'ELEVA2'/
C
      IERR = 0
C
      if (DLONG.lt.-DPI .or. DLONG.gt.D2PI) then
        IERR = 2
        call WRERR(RTNAME,IERR,'Error in the input parameter DLONG')
      endif
C
      if (DALPHA.lt.0.D0 .or. DALPHA.gt.D2PI) then
        IERR = 2
        call WRERR(RTNAME,IERR,'Error in the input parameter DALPHA')
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate the hour angle:
C
        DLT = DLAT
        DD = DDELTA
        DHG = DHGHT
        LRF = LREF
        call ELEVA1(DLT,DD,DHG,LRF,DHR,DAZ,ISTAT)
C
        if  (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        endif
C
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate time of nearest transit:
C
        if (dabs(DHR-DPI).lt.1.D-6) then
          DHR = 0.D0
          DLSID = DALPHA + DPI
        else
          DLSID = DALPHA
        endif
C
        LMEAN = .false.
        DJ = DJD
        DLG = DLONG
        call LSTJD1(DJ,DLG,DLSID,LMEAN,DJD0,ISTAT)
C
        if  (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,20)
        else
C
          DT = DHR / ( 1.002737908D0 * D2PI )
C
          DJD1 = DJD0 - DT
          DAZ1 = DAZ
C
          DJD2 = DJD0 + DT
          DAZ2 = D2PI - DAZ
C
        endif
      endif
C
C
      RETURN
      END
