! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ELSUN1(DJD,DLONG,DLAT,DHGHT,LREF,
     &                  DJD1,DAZ1,DJD2,DAZ2,IERR)
C
CC    Calculates the times corresponding to a given elevation of the sun
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:01
C
CA  author : KOD        date: 03-OCT-1989 19:04
CU  update : KOD        date: 30-MAR-1990 16:05
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the times (and corresponding azimuths)
CG  during which the true or apparent elevation (corrected for
CG  atmospheric refraction) of the sun EXCEEDS a given value. The
CG  apparent movement of the sun and the topocentric parallax will
CG  be taken into account.
CG
CG  If the elevation cannot be reached the times will be calculated
CG  for the corresponding transit and a warning message will be
CG  given ( IERR = 1 ) .  The azimuth is defined to start from
CG  south and increase towards west.
CG
CG  According to a resolution of the IAU east longitudes are counted
CG  positive, west long. negative (Sky and Tel, June, 1984, p.559).
CG
CGT   To determine the times for apparent rising or setting of the sun,
CGT   take an apparent elevation of  -13' = -0.004
CGT          or a true elevation of  -50' = -0.015 .
CG
CG  The times are given in Julian Ephemeris Times (approx. UT) and
CG  are selected to be as close to the given date as possible.
CG
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Day to be analysed
CP
CP  DLONG               R8  I   geographical longitude    (radians)
CP  DLAT                R8  I   geographical latitude     (radians)
CP
CP  DHGHT               R8  I   height above the horizon  (radians)
CP
CP  LREF                L4  I   correction for atmospheric refraction:
CP                                LREF = .true.  : with correction
CP                                LREF = .false. : without correction
CP
CP  DJD1                R8    O J.D. for ascending        ( < DJD2 )
CP  DAZ1                R8    O azimuth for DJD1          (radians)
CP
CP  DJD2                R8    O J.D. for descending       ( > DJD1 )
CP  DAZ2                R8    O azimuth for DJD2          (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
C
C   routines_called    type     description
CR  CSUN01              SR      calculates the position of the sun
CR  EQUECL              SR      converts equat. and ecl. coordinates
CR  ELEVA1              SR      determines the hour angle to an elev.
CR  ELEVA2              SR      determines the times to an elevation
CR  DISTP1              SR      calculates the 'parallactic distance'
CR  PARLX1              SR      corrects for topocentric parallax
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
C
      REAL*8        DJD, DLONG, DLAT, DHGHT, DJD1, DAZ1, DJD2, DAZ2,
     &              DDIST, DALPHA, DDELTA, DJD0, DA1, DA2, DEPS,
     &              DLSUN, DBSUN, DJ1, DJ2, DPRLX, DHR, DAZ, DALPH0,
     &              DDELT0, DJ, DLG, DLT, DHG
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT, IT, ITMAX
      LOGICAL       LMEAN, LDIR, LCONV, LSTOP, LREF, LPAR, LRF
      CHARACTER*8   RTNAME
C
      PARAMETER ( ITMAX = 10 )
      PARAMETER ( DEPS = 1.D-12 )
C
      DATA          RTNAME /'ELSUN1'/
C
      IERR = 0
C
C     calculate the apparent position of the sun for DJD:
C
      LMEAN = .false.
      DJ = DJD
      call CSUN01(DJ,LMEAN,DLSUN,DDIST,ISTAT)
C
      if  (ISTAT.gt.1) then
        IERR = max(IERR,ISTAT)
        call ERRSTC(RTNAME,10)
      endif
C
      if (IERR.lt.2) then
C
        LDIR = .false.
        DBSUN = 0.D0
        call EQUECL(DJ,LMEAN,LDIR,DLSUN,DBSUN,DALPHA,DDELTA,ISTAT)
C
        if  (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,20)
        endif
C
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate the parallax:
C
        call DISTP1(DDIST,DPRLX,ISTAT)
C
        if  (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,30)
        else
C
C         calculate the hour angle:
C
          DLT = DLAT
          DHG = DHGHT
          LRF = LREF
          call ELEVA1(DLT,DDELTA,DHG,LRF,DHR,DAZ,ISTAT)
C
          if  (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,40)
          endif
C
        endif
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate the approx. transition times for the elevation:
C
        DLG = DLONG
        call ELEVA2(DJ,DLG,DLT,DALPHA,DDELTA,DHG,LRF,
     &              DJ1,DA1,DJ2,DA2,ISTAT)
        DJD1 = DJ1
        DJD2 = DJ2
C
        if  (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,50)
        endif
C
C
C       improve the ascending position of the sun:
C
        LCONV = .false.
        LSTOP = .false.
        LPAR = .true.
        IT = 0
        do while (.not.LCONV .and. IT.lt.ITMAX .and. .not.LSTOP)
          IT = IT + 1
          DJD0 = DJD1
          call CSUN01(DJD0,LMEAN,DLSUN,DDIST,ISTAT)
          LSTOP = (ISTAT.gt.1)
          call EQUECL(DJD0,LMEAN,LDIR,DLSUN,DBSUN,DALPH0,DDELT0,ISTAT)
          LSTOP = (ISTAT.gt.1)
          call PARLX1(DLT,DHG,DPRLX,-DHR,DALPH0,DDELT0,LPAR,
     &                  DALPHA,DDELTA,ISTAT)
          LSTOP = (ISTAT.gt.1)
          call ELEVA2(DJD0,DLG,DLT,DALPHA,DDELTA,DHG,LRF,
     &                DJ1,DA1,DJ2,DA2,ISTAT)
          DJD1 = DJ1
          DAZ1 = DA1
          LSTOP = (ISTAT.gt.1)
          LCONV = (abs(DJD1-DJD0).lt.DEPS)
        end do
C
        if (LSTOP) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,60)
        endif
C
C
C       improve the descending position of the sun:
C
        LCONV = .false.
        LSTOP = .false.
        LPAR = .true.
        IT = 0
        do while (.not.LCONV .and. IT.lt.ITMAX .and. .not.LSTOP)
          IT = IT + 1
          DJD0 = DJD2
          call CSUN01(DJD0,LMEAN,DLSUN,DDIST,ISTAT)
          LSTOP = (ISTAT.gt.1)
          call EQUECL(DJD0,LMEAN,LDIR,DLSUN,DBSUN,DALPH0,DDELT0,ISTAT)
          LSTOP = (ISTAT.gt.1)
          call PARLX1(DLT,DHG,DPRLX,DHR,DALPH0,DDELT0,LPAR,
     &                  DALPHA,DDELTA,ISTAT)
          LSTOP = (ISTAT.gt.1)
          call ELEVA2(DJD0,DLG,DLT,DALPHA,DDELTA,DHG,LRF,
     &                DJ1,DA1,DJ2,DA2,ISTAT)
          DJD2 = DJ2
          DAZ2 = DA2
          LSTOP = (ISTAT.gt.1)
          LCONV = (abs(DJD2-DJD0).lt.DEPS)
        end do
C
        if (LSTOP) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,70)
        endif
C
      endif
C
C
      RETURN
      END
