! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine EQUHOR(DJD,LMEAN,DLONG,DLAT,LDIR,DLOLD,DBOLD,
     &                  DLNEW,DBNEW,IERR)
C
CC    Converts coordinates between the equatorial and horizontal system
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:02
C
CA  author : KOD        date: 05-SEP-1989 16:45
CU  update : KOD        date: 29-MAR-1990 15:21
CU  update : KOD        date: 01-FEB-1995           (dradrs)
C
CT  status: not tested
C
C   general description
CG  Subroutine to convert coordinates between the equatorial and the
CG  horizontal system (the azimuthal angle starting from south and
CG  running to west).
CG
CG  For the equatorial coordinates the current equinox should be used.
CG  (APPARENT positions are affected by aberration and nutation.)
CG  The height above the horizon { is assumed to be / will be }
CG  corrected for atmospheric refraction.
CG
CG  According to a resolution of the IAU east longitudes are counted
CG  positive, west long. negative (Sky and Tel, June, 1984, p.559).
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Date
CP
CP  LMEAN               L4  I   mean or apparent positions:
CP                                LMEAN = .true.  :  mean    positions
CP                                LMEAN = .false. : apparent positions
CP
CP  DLONG               R8  I   geographical longitude ( radians ! )
CP  DLAT                R8  I   geographical latitude  ( radians ! )
CP
CP  LDIR                L4  I   transformation direction:
CP                                LDIR = .true.  : equ --> hor
CP                                LDIR = .false. : equ <-- hor
CP
CP  DLOLD               R8  I   old longitude / azimuth    (radians)
CP  DBOLD               R8  I   old latitude / elevation   (radians)
CP
CP  DLNEW               R8    O new longitude / azimuth    (radians)
CP  DBNEW               R8    O new latitude / elevation   (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  REFRA1              SR      corrects for atmospheric refraction
CR  JDLST1              SR      calculates local siderial time
CR  EULER1              SR      general coordinate transformation
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DJD, DLONG, DLAT, DLOLD, DBOLD, DLNEW, DBNEW,
     &              DRADRS, DLSID, D1, D2, D3, DPHI1, DTHETA1,
     &              DPHI2, DTHETA2, DBLD, DBNW, DJ, DLNG, DB
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LMEAN, LDIR, LCONV, LMN
      CHARACTER*8   RTNAME
      logical       lfirst
C
      DATA          RTNAME /'EQUHOR'/
      data lfirst / .true. /
C
      IERR = 0
C
      if (DLOLD.lt.0.D0 .or. DLOLD.gt.D2PI .or. dabs(DBOLD).gt.DPID2)
     &  then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Error in input parameters DLOLD, DBOLD')
      endif
C
C     calculate the local sidereal time:
C
      DJ = DJD
      LMN = LMEAN
      DLNG = DLONG
      call JDLST1(DJ,LMN,DLNG,DLSID,ISTAT)
C
      if (ISTAT.gt.1) then
        IERR = max(IERR,ISTAT)
        call ERRSTC(RTNAME,10)
      endif
C
      if (IERR.lt.2) then
C
C       correct for atmospheric refraction:
C
        if (lfirst) then
          write(*,*) '+++ equhor: atmospheric refraction changed +++'
c          write(*,*) '+++ equhor: atm. refraction not changed +++'
          lfirst = .false.
        endif
        if (LDIR .or. DBOLD.lt.0.d0) then
          DBLD = DBOLD
        else
          LCONV = .true.
          DB = DBOLD
****          call REFRA1(DB,LCONV,DBLD,ISTAT)
          dbld = db  !@@@
C
          if (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,20)
          endif
C
        endif
      endif
C
      if (IERR.lt.2) then
C
        D1 =  dradrs(DLSID + DPID2)
        D2 =  DPID2 - DLAT
        D3 = -DPID2
C
        if (ldir) then
          DPHI1 = DLOLD
        else
          dphi1 = d2pi - dlold
        endif
C
        DTHETA1 = DPID2 - DBLD
C
        if (LDIR) then
           call EULER1(D1,D2,D3,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
        else
           call EULER1(-D3,-D2,-D1,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
        endif
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,30)
        endif
      endif
C
      if (IERR.lt.2) then
C
        if (ldir) then
          DLNEW = D2PI - DPHI2
        else
          dlnew = dphi2
        endif
C
        DBNW  = DPID2 - DTHETA2
C
C       correct for atmospheric refraction:
C
        if (.not.LDIR .or.dbnw.lt.0.d0) then
          DBNEW = DBNW
        else
          LCONV = .false.
****          call REFRA1(DBNW,LCONV,DBNEW,ISTAT)
          dbnew = dbnw   !@@@@
C
          if (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,40)
          endif
C
        endif
      endif
C
      RETURN
      END
