! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine GREJUL(LDIR,IYOLD,IMOLD,IDOLD,IYNEW,IMNEW,IDNEW,IERR)
C
CC    Converts between the Gregorian and Julian Calendar system
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:03
C
CA  author : KOD        date: 23-AUG-1989 13:56
CU  update : KOD        date: 20-MAR-1990 14:59
C
CT  status: tested
C
C   general description
CG  Subroutine to convert between the Gregorian and Julian Calendar
CG  system. The input values will be checked for validity.
CG
CG  This subroutine is valid for the years  -4712  to  +3500 ,
CG  (cf. 'Sky and Telescope, May 1984, p.455) and has been written
CG  according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.24,26.
C
C   call_var.          type I/O description
CP  LDIR                L4  I   direction of conversion
CP                                LDIR = .true.  :  Greg --> Jul
CP                                LDIR = .false. :  Greg <-- Jul
CP
CP  IYOLD               I4  I   'old' year          (-4712 .. 3500)
CP  IMOLD               I4  I   'old' month         (1..12)
CP  IDOLD               I4  I   'old' day of month  (1..28,29,30,31)
CP
CP  IYNEW               I4    O 'new' year          (-4712 .. 3500)
CP  IMNEW               I4    O 'new' month         (1..12)
CP  IDNEW               I4    O 'new' day of month  (1..28,29,30,31)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$INCLUDE:ICGENL.INC        general parameter common block
C
C   routines_called    type     description
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
C
      REAL*8        DH
      REAL*4        HFLAG
      INTEGER*4     IERR, IOS, IYOLD, IMOLD, IDOLD, IYNEW, IMNEW,
     &              IDNEW, IYY, IMM, JD, IGREG, I, IA, IB, IC, IG, IE,
     &              IY, IM, ID
      LOGICAL*4     LDIR, LCHECK
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'GREJUL'/
C
      IERR=0
C
C     rough input check:
C
      if ( (.not.LDIR .and. IYOLD .lt.-4712) .or.
     &     IYOLD .gt.3500 .or.
     &     IMOLD .lt.1     .or. IMOLD .gt.12   .or.
     &     IDOLD .lt.1     .or. IDOLD .gt.31 )  then
C
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input parameters')
C
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate the Julian Date:
C
        if (IMOLD.gt.2) then
          IYY = IYOLD
          IMM = IMOLD
        else
          IYY = IYOLD - 1
          IMM = IMOLD + 12
        endif
C
        if (IYY.gt.0) then
          DH=0.D0
        else
          DH=0.75D0
        endif
C
        JD = idint(365.25d0*dble(IYY)-DH) +
     &       idint(30.6001d0*dble(IMM+1)) +
     &       IDOLD + 1720995
C
        if (LDIR) then
          IGREG = IYY/100
          IGREG = 2 - IGREG + IGREG/4
          JD = JD + IGREG
        endif
C
        if (JD.lt.0) then
          IERR = max(IERR,2)
          call WRERR(RTNAME,IERR,'Error in input parameters')
        endif
C
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate the Calendar Date:
C
        do I = 1, 2
C
          LCHECK = (I.eq.1)
           if (LCHECK.eqv.LDIR) then
             IA = idint( (dble(JD)-1867216.25D0) / 36524.25D0 )
             IA = JD + 1 + IA - IA/4
           else
             IA = JD
           endif
C
           IB = IA + 1524
           IC = idint( (dble(IB)-122.1D0) / 365.25D0 )
           IG = idint( 365.25D0 * dble(IC) )
           IE = idint( dble(IB-IG) / 30.6001D0 )
C
           ID = IB - IG - int( 30.6004 * real(IE) )
C
           if (IE.le.13) then
             IM = IE - 1
           else
             IM = IE - 13
           endif
C
           if (IM.ge.3) then
             IY = IC - 4716
           else
             IY = IC - 4715
           endif
C
          if (LCHECK) then
            if ( IY.ne.IYOLD .or. IM.ne.IMOLD .or. ID.ne.IDOLD )  then
               IERR = max(IERR,2)
               call WRERR(RTNAME,IERR,'input date invalid')
            endif
          else
            IYNEW = IY
            IMNEW = IM
            IDNEW = ID
          endif
C
        end do
C
      endif
C
      RETURN
      END
