! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine KMOON1(DJD,DPHANG,DFRC,IERR)
C
CC    Calc. the phase angle and the illum. fract. of the moon's disk
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:07
C
CA  author : KOD        date: 05-OCT-1989 14:18
CU  update : KOD        date: 29-MAR-1990 16:02
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the (geocentric) phase angle and the
CG  illuminated fraction of the moon for a given date, according to
CG
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                  third edition, 1985, p.155-157 .
CG
CG  The phase angle is defined as the angular distance sun-earth
CG  as seen from the moon.
C
C   call_var.          type I/O description
CP  DJD                 R8  I   Julian Date  (Ephemeris time)
CP
CP  DPHANG              R8    O phase angle  (radians)
CP
CP  DFRC                R8    O illuminated fraction of the moon's disk
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  CSUN01              SR      calculates the position of the sun
CR  CMOON1              SR      calculates the position of the moon
CR  DRADRS              R8      'resets' radians
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DJD, DPHANG, DFRC, DLSUN, DDIST, DLMON, DBMON,
     &              DPRLX, DJD0, DYEAR, DT, DRADRS, DMS, DMM,
     &              DDMS(4), DDMM(4), DH, DH1, DH2, DJ
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LMEAN
      CHARACTER*8   RTNAME
C
      PARAMETER ( DJD0 = 2415020.D0 )
      PARAMETER ( DYEAR =  36525.D0 )
C
      DATA          RTNAME /'KMOON1'/
C
      DATA DDMS / 358.475833D0,  35999.0498D0,-0.000150D0,-0.0000033D0 /
      DATA DDMM / 296.104608D0, 477198.8491D0, 0.009192D0, 0.0000144D0 /
C
      IERR = 0
C
C     calculate the ecliptical position of the sun:
C
      LMEAN = .true.
      DJ = DJD
      call CSUN01(DJ,LMEAN,DLSUN,DDIST,ISTAT)
C
      if  (ISTAT.gt.1) then
        IERR = max(IERR,ISTAT)
        call ERRSTC(RTNAME,10)
      endif
C
      if (IERR.lt.2) then
C
C       calculate the ecliptical position of the moon:
C
        call CMOON1(DJ,DLMON,DBMON,DPRLX,ISTAT)
C
        if  (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,20)
        endif
C
      endif
C
      if (IERR.lt.2) then
C
C       calculate the mean anomalies of the sun and moon:
C
        DT = (DJD - DJD0) / DYEAR
C
        DMS = DDMS(1) + (DDMS(2) + (DDMS(3) + DDMS(4) * DT) * DT) * DT
        DMM = DDMM(1) + (DDMM(2) + (DDMM(3) + DDMM(4) * DT) * DT) * DT
C
        DMS = DMS * DPI180
        DMM = DMM * DPI180
C
        DMS = DRADRS(DMS)
        DMM = DRADRS(DMM)
C
C       calculate the phase angle:
C
        DH = acos( cos(DLMON-DLSUN) * cos(DBMON) )
C
        DH1 = 1.D0 - 0.0549D0 * sin(DMM)
        DH2 = 1.D0 - 0.0167D0 * sin(DMS)
C
        DH = 180.D0 - DH * D180PI - 0.1468D0 * sin(DH) * DH1 / DH2
C
        DH = DH * DPI180
        DPHANG = DRADRS(DH)
C
C       calculate the illuminated fraction:
C
        DFRC = 0.5D0 * ( 1.D0 + cos(DPHANG) )
C
      endif
C
      RETURN
      END
