! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine LSTJD1(DJD0,DLONG,DLSID,LMEAN,DJD,IERR)
C
CC    Calculates the time from the mean/apparent local sid.time
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:07
C
CA  author : KOD        date: 05-SEP-1989 16:10
CU  update : KOD        date: 29-MAR-1990 16:09
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the Universal Time corresponding
CG  to a given mean/apparent local sidereal time. This time
CG  will be selected to be nearest to the given date.
CG
CG  For consistency with the other subroutines the sidereal time
CG  and geographical longitude are required in radians.
CG
CG  According to a resolution of the IAU east longitudes are counted
CG  positive, west long. negative (Sky and Tel, June, 1984, p.559).
C
C   call_var.          type I/O description
CP  DJD0                R8  I   Julian Date (nearest time)
CP
CP  DLONG               R8  I   geographical longitude ( radians ! )
CP
CP  DLSID               R8  I   local sidereal time ( radians ! )
CP
CP  LMEAN               L4  I   mean or app. sidereal time:
CP                                LMEAN = .true.  : mean sid. time
CP                                LMEAN = .false. : app. sid. time
CP
CP  DJD                 R8    O Julian Date corresponding to DSID
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$INCLUDE:ICGENL.INC        general parameter common block
CI  R$INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description
CR  JDLST1              SR      calc. the local mean/app. sid. time
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
c*
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
      REAL*8        DPI, D2PI, DPID2, DPI180, D180PI     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI    =  3.141592653589793238D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( D2PI   =  6.283185307179586477D0 )     ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPID2  =  1.5707963267948966192D0 )    ! R$INCLUDE:IGENL.INC
      PARAMETER ( DPI180 =  0.017453292519943295769D0 )  ! R$INCLUDE:IGENL.INC
      PARAMETER ( D180PI =  57.29577951308232088D0 )     ! R$INCLUDE:IGENL.INC
c     ---------------------------------------------------! R$INCLUDE:IGENL.INC
C
      REAL*8        DJD0, DLONG, DLSID, DSID, DRADRS, DLSID0, DJD, DT,
     &              DJ0, DLG
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LMEAN, LMN
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'LSTJD1'/
C
      IERR = 0
C
      if (DSID.lt.0.D0 .or. DSID.gt.D2PI) then
        IERR = 1
        call WRERR(RTNAME,IERR,'Error in the input parameter DSID')
        DSID = DRADRS(DSID)
      endif
C
      if (IERR.lt.2) then
C
C       calculate the local sidereal time at DJD0:
C
        DJ0 = DJD0
        LMN = LMEAN
        DLG = DLONG
        call JDLST1(DJ0,LMN,DLG,DLSID0,ISTAT)
C
        if  (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        endif
C
      endif
C
      if (IERR.lt.2) then
C
C       calculate the difference in local sidereal time:
C
        DT = DRADRS ( DLSID - DLSID0 ) / D2PI
C
        if (DT.gt.0.5D0) DT = DT - 1.D0
C
C       convert the difference into solar time:
C
        DT = DT / 1.002737908D0
C
        DJD = DJD0 + DT
C
      endif
C
C
      RETURN
      END
