! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ANGCQ1(LEQU,DL,DB,DEPS,DANGCQ,IERR)
C
CC    Calculates the angle between eclipt. and equat. north directions
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:33
C
CA  author : KOD        date: 22-JUN-1990 13:19
CU  update : KOD        date: 26-JUN-1990 17:04
CU  update : KOD        date: 17-AUG-1992 10:00 (`1.D-6')
C
CT  status: tested
C
C   general description 
CG
CG  Subroutine to calculate the angle between the ecliptical and
CG  equatorial north meridians for a specified direction on the sky.
CG  This angle is defined counterclockwise from the equatorial to
CG  the ecliptical north meridian (as viewed from the inside of
CG  a sphere, as usual) and will be in the interval (0,2*pi).
CG  The direction on the sky can be supplied in equatorial or
CG  ecliptical coordinates. Use the subroutine JDEPS1 to calculate
CG  the obliquity of the ecliptic.
CG  This subroutine has been optimized for high speed.
C
C   call_var.          type I/O description 
CP  LEQU                L4  I   equatorial or ecliptical ccordinates:
CP                                LEQU = .true.  : equatorial coord.
CP                                LEQU = .false. : ecliptical coord.
CP  DL                  R8  I   R.A. or ecliptical longitude (radians)
CP  DB                  R8  I   Dec. or ecliptical latitude  (radians)
CP  DEPS                R8  I   obliquity of the ecliptic    (radians)
CP  DANGCQ              R8    O angle between the meridians  (radians)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
CI  R_INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  WRERR               SR      formats error code in standard form
CR  DRADRS              R8      `resets' radians
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
      INCLUDE 'R_INCLUDE:IGENL.INC'
C
      REAL*8        DL, DB, DEPS, DANGCQ, DX, DY, DH, DRADRS,
     &              DSE, DCE, DSA, DCA, DSD, DCD, DSL, DCL, DSB, DCB,
     &              DLM, DBT, DAL, DDE, DANG
      REAL*4        HFLAG
      INTEGER*4     IERR, IOS
      LOGICAL*4     LEQU
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'ANGCQ1'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
C     check input values:
C
      if (DL.lt.-1.D-6 .or. DL.GT.D2PI+1.D-6) then
        IERR = 1
        call WRERR(RTNAME,IERR,'Warning: unusual value for DL')
      endif
C
      if (dabs(DB).gt.DPID2+1.D-6) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input parameter DB')
      endif
C
C
      if (IERR.lt.2) then
C
        DSE = dsin(DEPS)
        DCE = dcos(DEPS)
C
        if (LEQU) then
C
          DSA = dsin(DL)
          DCA = dcos(DL)
          DSD = dsin(DB)
          DCD = dcos(DB)
C
          DX = DCD * DCA
          DY = DCE * DCD * DSA + DSE * DSD
C
          DLM = datan2(DY,DX)
C
          DSL = dsin(DLM)
          DCL = dcos(DLM)
C
        else
C
          DSL = dsin(DL)
          DCL = dcos(DL)
          DSB = dsin(DB)
          DCB = dcos(DB)
C
          DX = DCB * DCL
          DY = DCE * DCB * DSL - DSE * DSB
C
          DAL = datan2(DY,DX)
C
          DSA = dsin(DAL)
          DCA = dcos(DAL)
C
        endif
C
        DANG = dacos ( DSL * DSA + DCL * DCA * DCE )
C
        DH = DRADRS(DL+DPID2) - DPI
        if (DH.lt.0.D0) DANG = -DANG
C
        DANGCQ = DRADRS(DANG)
C
      endif
C
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
