! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ANGPOS(DALPH0,DDELT0,DALPHA,DDELTA,DPOS,IERR)
C
CC    Calculates the position angle
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:33
C
CA  author : KOD        date: 06-FEB-1991 19:24
CU  update : KOD        date: 19-FEB-1991 18:21
C
CT  status: tested
C
C   general description 
CG
CG  Subroutine to calculate the position angle between two positions.
CG  This value is defined as the angle between the great circle
CG  passing through both positions and the local north meridian as
CG  seen from the position (DALPH0, DDELT0). It is counted
CG  counterclockwise from North over East, South, West to North.
C
C   call_var.          type I/O description 
CP  DALPH0              R8  I   Right Ascension of reference  (radians)
CP  DDELT0              R8  I   Declination     of reference  (radians)
CP  DALPHA              R8  I   Right Ascension               (radians)
CP  DDELTA              R8  I   Declination                   (radians)
CP  DPOS                R8    O position angle                (radians)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
CI  R_INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  EULER1              SR      general coordinate transformation
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
CR  DRADRS              R8      'resets' radians
CR  ERRSTC              SR      continue error stack for one level
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
      INCLUDE 'R_INCLUDE:IGENL.INC'
C
      REAL*8        DALPH0, DDELT0, DALPHA, DDELTA, DPOS,
     &              D1, D2, D3, DPH1, DTH1, DPH2, DTH2, DRADRS
      REAL*4        HFLAG
      INTEGER*4     IERR, IOS, ISTAT
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'ANGPOS'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
      if (max(abs(DALPH0),abs(DALPHA)).gt.D2PI) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Unusual value for the R.A.')
      endif
C
      if (max(abs(DDELT0), abs(DDELTA)).gt.DPID2) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input parameters')
      endif
C
C
      if (IERR.lt.2) then
C
        D1 =  DRADRS ( DPID2 + DALPH0 )
        D2 =  DPID2 - DDELT0
        D3 =  0.D0
C
        DPH1 = DALPHA
        DTH1 = DPID2 - DDELTA
C
        call EULER1(D1,D2,D3,DPH1,DTH1,DPH2,DTH2,ISTAT)
C
        if (ISTAT.gt.0) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
C
        else
C
          DPOS = DRADRS ( DPID2 - DPH2 )
C
        endif
C
      endif
C
C
D     IF (HFLAG(RTNAME,'out: !<output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
