! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine CPREC1(DJD1,DJD2,DLOLD,DBOLD,DLNEW,DBNEW,IERR)
C
CC    Converts equatorial coordinates from one equinox to another
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:34
C
CA  author : KOD        date: 26-SEP-1989 15:55
CU  update : KOD        date: 28-MAR-1990 20:03
CU  update : KOD        date: 17-AUG-1992 10:04 (`1.D-6')
C
CT  status: tested
C
C   general description 
CG  Subroutine to convert the equatorial coordinates from one
CG  equinox to another, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.65,66.
CG
CGT Notes: the effect of the proper motion must be calculated BEFORE
CGT        converting the coordinates to another epoch.
CGT
CGT        The equinox of 1900.0 corresponds to JD 2415020.313,
CGT        the equinox of 1950.0 corresponds to JD 2433282.423,
CGT        according to J.Meeus, p.65 .
CGT        The equinox of 2000.0 corresponds to JD 2451545.000.
C
C   call_var.          type I/O description 
CP  DJD1                R8  I   Julian Date for the initial epoch
CP  DJD2                R8  I   Julian Date for the  final  epoch
CP  DLOLD               R8  I   old longitude  (radians)
CP  DBOLD               R8  I   old latitude   (radians)
CP  DLNEW               R8    O new longitude  (radians)
CP  DBNEW               R8    O new latitude   (radians)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
CI  R_INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  CVEQUI              SR      calculates quantities for epoch conv.
CR  REPOL1              SR      rectangular to polar coord. conversion
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
      INCLUDE 'R_INCLUDE:IGENL.INC'
C
      REAL*8        DJD1, DJD2, DLOLD, DBOLD, DLNEW, DBNEW, DZETA, DZ,
     &              DTHETA, DL, DCL, DSL, DCB, DSB, DCT, DST, DA, DB,
     &              DC, DH1, DR, DPHI, DH2, DRADRS, DJ1, DJ2
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'CPREC1'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
      if (DLOLD.lt.-1.D-6 .or. DLOLD.gt.D2PI+1.D-6) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Warning: unusual input value for DLOLD')
      endif
C
      if (dabs(DBOLD).gt.DPID2+1.D-6) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input parameter DBOLD')
      endif
C
      if (IERR.lt.2) then
        DJ1 = DJD1
        DJ2 = DJD2
        call CVEQUI(DJ1,DJ2,DZETA,DZ,DTHETA,ISTAT)
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        endif
C
      endif
C
      if (IERR.lt.2) then
C
        DL = DLOLD + DZETA
        DCL = cos(DL)
        DSL = sin(DL)
C
        DCB = cos(DBOLD)
        DSB = sin(DBOLD)
C
        DCT = cos(DTHETA)
        DST = sin(DTHETA)
C
        DA = DCB * DSL
        DB = DCT * DCB * DCL - DST * DSB
        DC = DST * DCB * DCL + DCT * DSB
C
        DH1 = 0.D0
        call REPOL1(DB,DA,DH1,DR,DPHI,DH2)
        DLNEW = DRADRS(DPHI + DZ)
C
        DBNEW = asin(DC)
        if (abs(DBNEW).gt.0.5D0*DPID2) DBNEW = sign(acos(DR),DBNEW)
C
      endif
C
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
