! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine CPROP1(DJD0,DJD,DLOLD,DBOLD,DLP,DBP,DLNEW,DBNEW,IERR)
C
CC    Calculates the change of the equ. coordinates due to proper motion
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:34
C
CA  author : KOD        date: 26-SEP-1989 16:50
CU  update : KOD        date: 20-MAR-1990 14:42
CU  update : KOD        date: 17-AUG-1992 10:08 (`1.D-6')
C
CT  status: tested
C
C   general description 
CG  Subroutine to calculate the effect of proper motion on the
CG  equatorial coordinates. This effect has to be taken into
CG  account BEFORE converting the coordinates to another epoch,
CG  according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.66 .
C
C   call_var.          type I/O description 
CP  DJD0                R8  I   Julian Date for the initial epoch
CP  DJD                 R8  I   Julian Date for the current epoch
CP  DLOLD               R8  I   old longitude  (radians)
CP  DBOLD               R8  I   old latitude   (radians)
CP  DLP                 R8  I   annual proper motion in R.A. (radians)
CP  DBP                 R8  I   annual proper motion in Dec. (radians)
CP  DLNEW               R8    O new longitude  (radians)
CP  DBNEW               R8    O new latitude   (radians)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
CI  R_INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
      INCLUDE 'R_INCLUDE:IGENL.INC'
C
      REAL*8        DJD0, DJD, DLOLD, DBOLD, DLP, DBP, DLNEW, DBNEW, DT
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'CPROP1'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
      if (DLOLD.lt.-1.D-6 .or. DLOLD.gt.D2PI+1.D-6) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Warning: unusual input value for DLOLD')
      endif
C
      if (dabs(DBOLD).gt.DPID2+1.D-6) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in the input parameter DBOLD')
      endif
C
      if ( max( abs(DLP*cos(DBOLD)), abs(DBP) ).gt.1.D-4) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input parameters DLP, DBP')
      endif
C
      if (min(DJD0,DJD).lt.0.D0 .or. max(DJD0,DJD).gt.3.D6) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in the input parameter DJD')
      endif
C
      if (IERR.lt.2) then
        DT = (DJD - DJD0) / 365.242199D0
        DLNEW = DLOLD + DT * DLP
        DBNEW = DBOLD + DT * DBP
      endif
C
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
