! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine CVEQUI(DJD1,DJD2,DZETA,DZ,DTHETA,IERR)
C
CC    Calculates quantities needed for equinox conversions
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:34
C
CA  author : KOD        date: 26-SEP-1989 15:09
CU  update : KOD        date: 20-MAR-1990 14:44
C
CT  status: tested
C
C   general description 
CG  Subroutine to calculate the quantities needed for the accurate
CG  reduction of positions from one equinox to another,
CG  according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.65 .
C
C   call_var.          type I/O description 
CP  DJD1                R8  I   Julian Date for the initial epoch
CP  DJD2                R8  I   Julian Date for the  final  epoch
CP  DZETA               R8    O 'zeta'   (radians)
CP  DZ                  R8    O 'z'      (radians)
CP  DTHETA              R8    O 'theta'  (radians)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
C
C   routines_called    type     description 
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
C
      REAL*8        DJD1, DJD2, DZETA, DZ, DTHETA, DJD0, DYEAR,
     &              DT0, DT, DH, DF
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS
      CHARACTER*8   RTNAME
C
      PARAMETER ( DJD0  = 2415020.313D0  )
      PARAMETER ( DYEAR =  36524.2199D0  )
      PARAMETER ( DF    = 4.848136811D-6 )
C
      DATA          RTNAME /'CVEQUI'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
      if (min(DJD1,DJD2).lt.0.D0 .or. max(DJD1,DJD2).gt.3.D6) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in the input parameter DJD')
      endif
C
      if (IERR.lt.2) then
C
        DT0 = ( DJD1 - DJD0 ) / DYEAR
        DT  = ( DJD2 - DJD1 ) / DYEAR
C
        DH = 2304.250D0 + 1.396D0 * DT0
        DZETA = ( DH + ( 0.302D0 + 0.018D0 * DT ) * DT ) * DT
C
        DZ = DZETA + ( 0.791D0 + 0.001D0 * DT ) * DT * DT
C
        DH = 2004.682D0 - 0.853D0 * DT0
        DTHETA = ( DH - ( 0.426D0 + 0.042D0 * DT ) * DT ) * DT
C
        DZETA  = DZETA  * DF
        DZ     = DZ     * DF
        DTHETA = DTHETA * DF
C
      endif
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
