! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ERRBX1(DL0,DB0,DBW,DBH,DBT,DDL,DDB,IERR)
C
CC    Calculates the coordinates of the corners of an error box
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:35
C
CA  author : KOD        date: 28-JUN-1990 20:34
CU  update : KOD        date: 26-OCT-1990 18:23 (no warning in EULER1)
CU  update : KOD        date: 17-AUG-1992 10:11 (`1.D-6')
C
CT  status: tested
C
C   general description 
CG
CG  Subroutine to calculate the coordinates of the corners of an
CG  error box which is specified by its width, height, tilt, and
CG  the coordinates of its center. The height parameter determines
CG  the extent of the `vertical' boundaries, which are part of a
CG  great circle on the sphere, while the `horizontal' boundaries
CG  specified by the width parameter are determined as small circles
CG  of equal length on the sphere. The tilt of the error box is
CG  the angle between the symmetry plane of the width boundaries
CG  and the declination line (DB0) running through the center of
CG  the error box, measured counterclockwise. Note that the dimensions
CG  of the source box are required in `true' degrees (i.e., the values
CG  are independent of the declination) to be consistent with any
CG  tilts. The calculation is done in full spherical geometry.
C
C   call_var.          type I/O description 
CP  DL0                 R8  I   Right Ascension of the center  (radians)
CP  DB0                 R8  I   Declination of the center      (radians)
CP  DBW                 R8  I   full width  of the error box   (radians)
CP  DBH                 R8  I   full height of the error box   (radians)
CP  DBT                 R8  I   tilt of the error box          (radians)
CP  DDL                 R8    O Right Ascension of the corners (radians)
CP  DDB                 R8    O Declination     of the corners (radians)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
CI  R_INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  EULER1              SR      transformation of spherical coordinates
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to flag output stream
CR  DRADRS              R8      'resets' radians
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
      INCLUDE 'R_INCLUDE:IGENL.INC'
C
      REAL*8        DL0, DB0, DBW, DBH, DBT, DDL(4), DDB(4), D1, D2, D3,
     &              DPH1, DTH1, DPH2, DTH2, DW, DH, DRADRS
      REAL*4        HFLAG
      INTEGER*4     IERR, IOS, ISTAT, I
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'ERRBX1'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
C     check input values:
C
      if (max(abs(DL0),DBW,abs(DBT)).gt.D2PI+1.D-6) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Unusual input values')
      endif
C
      if ( abs(DB0).gt.DPID2+1.D-6 .or. DBH.gt.DPI .or.
     &     DBW.lt.0.D0 .or. DBH.lt.0.D0 ) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input values')
      endif
C
C
      if (IERR.lt.2) then
C
        do I = 1, 4
C
          if (I.le.2) then
            DW = D2PI - 0.5D0 * DBW
          else
            DW = 0.5D0 * DBW
          endif
C
          if (I.eq.1 .or. I.eq.4) then
            DH =  0.5D0 * DBH
          else
            DH = -0.5D0 * DBH
          endif
C
          D1 = 0.D0
          D2 =  DBT
          D3 = 0.D0
C
          DPH1 = DRADRS(DW)
          DTH1 = DPID2 - DH
C
          CALL EULER1(D1,D2,D3,DPH1,DTH1,DPH2,DTH2,ISTAT)
C
          if (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,10)
          endif
 
          if (IERR.lt.2) then
C
            D1 =  DPID2
            D2 =  DB0
            D3 = DRADRS(-DPID2-DL0)
C
            DPH1 = DPH2
            DTH1 = DTH2
C
            CALL EULER1(D1,D2,D3,DPH1,DTH1,DPH2,DTH2,ISTAT)
C
            if (ISTAT.gt.1) then
              IERR = max(IERR,ISTAT)
              call ERRSTC(RTNAME,20)
            else
              DDL(I) = DPH2
              DDB(I) = DPID2 - DTH2
            endif
C
          endif
C
        enddo
C
      endif
C
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
