! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine ETANOM(DECC,LDIR,DANOM1,DANOM2,IERR)
C
CC    Converts between the eccentric and true anomaly
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:36
C
CA  author : KOD        date: 25-SEP-1989 13:29
CU  update : KOD        date: 20-MAR-1990 15:40
C
CT  status: tested
C
C   general description 
CG    Subroutine to convert between the eccentric and true anomaly
CG    for an elliptic orbit, according to
CG       Jean Meeus, 'Astronomical Formulae for Calculators',
CG                   third edition, 1985, p.122 .
C
C   call_var.          type I/O description 
CP  DECC                R8  I   eccentricity  ( 0.0 <= DECC < 1.0 )
CP  LDIR                L4  I   conversion direction:
CP                                LDIR = .true.  : ecc --> true anom.
CP                                LDIR = .false. : ecc <-- true anom.
CP  DANOM1              R8  I   'old' anomaly  (radians)
CP  DANOM2              R8    O 'new' anomaly  (radians, 0 .. 2 * pi )
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
C
C   routines_called    type     description 
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
C
      REAL*8        DECC, DANOM1, DANOM2, DH1, DH2, DHS, DHC, DH,
     &              DRADRS
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS
      LOGICAL*4     LDIR
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'ETANOM'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
      if (dabs(DANOM1).gt.12.56D0) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Error in anomaly')
      endif
C
      if (DECC.lt.0.D0 .or. DECC.ge.1.D0) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in eccentricity')
      endif
C
      if (IERR.lt.2) then
C
        if (LDIR) then
          DH1 = sqrt ( (1.D0 + DECC) / (1.D0 - DECC) )
        else
          DH1 = sqrt ( (1.D0 - DECC) / (1.D0 + DECC) )
        endif
C
        DH2 = 0.5D0 * DANOM1
C
        DHS = sin(DH2)
        DHC = cos(DH2)
C
        DH = 2.D0 * atan2(DH1*DHS,DHC)
C
        DANOM2 = DRADRS(DH)
C
      endif
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
