! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine GEORCO(DPHI,DHT,DRSPHI,DRCPHI,IERR)
C
CC    Calculates the GEOcentric Rectangular COordinates of an observer
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:36
C
CA  author : KOD        date: 29-AUG-1989 19:16
CU  update : KOD        date: 30-MAR-1990 16:10
CU  update : KOD        date: 17-AUG-1992 11:15 (`1.D-6')
CU  update : PAUL       date: 16-NOV-1992 16:04 null charactes deleted
C
CT  status: tested
C
C   general description 
CG  Subroutine to calculate the geocentric rectangular coordinates
CG  of an observer, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.37-38 .
C
C   call_var.          type I/O description 
CP  DPHI                R8  I   geographical latitude  (radians)
CP  DHT                 R8  I   height above sea level  (meters)
CP  DRSPHI              R8    O rho * sin(geocentric latitude)
CP  DRCPHI              R8    O rho * cos(geocentric latitude)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
CI  R_INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
CR  WRERR               SR      formats error code in standard form
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
      INCLUDE 'R_INCLUDE:IGENL.INC'
C
      REAL*8        DPHI, DHT, DRSPHI, DRCPHI, DBA, DHT0, DU, DH
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS
      CHARACTER*8   RTNAME
C
      PARAMETER ( DBA   = 0.99664719D0 )
      PARAMETER ( DHT0  = 6378140.D0 )
C
      DATA          RTNAME /'GEORCO'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
      if (DHT.lt.-1.D2 .or. DHT.gt.1.D4) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'Error in the input parameter DHT')
      endif
      if (dabs(DPHI).gt.DPID2+1.D-6) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in the input parameter DPID2')
      endif
C
      if (IERR.lt.2) then
C
        if (abs(abs(DPHI)-DPID2).lt.1.D-6) then
          DU = sign(DPID2,DPHI)
        else
          DU = atan( DBA * tan(DPHI) )
        endif
C
        DH = DHT / DHT0
C
        DRSPHI = DBA * sin(DU) + DH * sin(DPHI)
        DRCPHI = cos(DU)       + DH * cos(DPHI)
C
      endif
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
