! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine JDLST1(DJD,LMEAN,DLONG,DLSID,IERR)
C
CC    Calculates the local mean/apparent sidereal time for a given JD
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:37
C
CA  author : KOD        date: 05-SEP-1989 16:40
CU  update : KOD        date: 29-MAR-1990 15:56
CU  update : KOD        date: 17-AUG-1992 09:57 (DPI+1.D-6)
C
CT  status: tested
C
C   general description 
CG
CG  Subroutine to calculate the local mean/apparent sidereal time
CG  from the Julian Date, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.39-41,44.
CG
CG  For APPARENT positions (affected by aberration and nutation)
CG  use the APPARENT sid. time, otherwise the MEAN sidereal time.
CG
CG  For consistency with the other subroutines radians are used for
CG  the geographical longitude and sidereal time.
CG
CG  According to a resolution of the IAU east longitudes are counted
CG  positive, west long. negative (Sky and Tel, June, 1984, p.559).
C
C   call_var.          type I/O description 
CP  DJD                 R8  I   Julian Date
CP  LMEAN               L4  I   mean or app. sidereal time:
CP                                LMEAN = .true.  : mean sid. time
CP                                LMEAN = .false. : app. sid. time
CP  DLONG               R8  I   geographical longitude ( radians ! )
CP  DLSID               R8    O local sidereal time    ( radians ! )
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
CI  R_INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  JDSID1              SR      calc. the sid. time at Greenwich
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
      INCLUDE 'R_INCLUDE:IGENL.INC'
C
      REAL*8        DJD, DLONG, DLSID, DSID, DRADRS, DJ
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LMEAN, LMN
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'JDLST1'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
      if (dabs(DLONG).gt.DPI+1.D-6) then
        IERR = 2
        call WRERR(RTNAME,IERR,'Error in the input parameter DLONG')
      endif
C
      if (IERR.lt.2) then
        DJ = DJD
        LMN = LMEAN
        call JDSID1(DJ,LMN,DSID,ISTAT)
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        else
          DLSID = DRADRS ( DSID + DLONG )
        endif
      endif
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
