! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine JDNUT1(DJD,DP,DE,IERR)
C
CC    Calculates the nutation in longitude and in obliquity from the JD
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:37
C
CA  author : KOD        date: 29.08L-1989 14:45
CU  update : KOD        date: 20-MAR-1990 15:04
C
CT  status: tested
C
C   general description 
CG  Subroutine to calculate the nutation in longitude and the
CG  nutation in obliquity which are needed for the calculation
CG  of the APPARENT place of a star and for that of the APPARENT
CG  sidereal time, according to
CG     Jean Meeus, 'Astronomical Formulae for Calculators',
CG                 third edition, 1985, p.69-70 .
C
C   call_var.          type I/O description 
CP  DJD                 R8  I   Julian Date
CP  DP                  R8    O nutation in longitude (radians)
CP  DE                  R8    O nutation in obliquity (radians)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
C
C   routines_called    type     description 
CR  WRERR               SR      formats error code in standard form
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
C
      REAL*8        DJD, DP, DE, DJD0, DYEAR, DT, DLS, DLM,
     &              DMS, DMM, DOM, DH, DLS2, DLM2, DOM2, DF
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS
      CHARACTER*8   RTNAME
C
      PARAMETER ( DJD0 = 2415020.D0 )
      PARAMETER ( DYEAR =  36525.D0 )
C
      PARAMETER ( DF = 4.848136811D-6 )
C
      DATA          RTNAME /'JDNUT1'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
      if (DJD.lt.0.D0 .or. DJD.gt.3.D6) then
        IERR = 2
        call WRERR(RTNAME,IERR,'Error in the input parameter DJD')
      endif
C
      if (IERR.lt.2) then
C
        DT = (DJD - DJD0) / DYEAR
C
        DLS = 4.8816283D0 + (628.3319506D0 + 5.2883D-6 * DT) * DT
        DLM = 4.7199672D0 + (8399.709144D0 - 1.9775D-5 * DT) * DT
        DMS = 6.2565830D0 + (628.3019466D0 - 2.6180D-6 * DT) * DT
        DMM = 5.1680002D0 + (8328.691104D0 + 1.6043D-4 * DT) * DT
        DOM = 4.5236020D0 - (33.75714610D0 + 3.6268D-5 * DT) * DT
C
        DLS2 = 2.D0 * DLS
        DLM2 = 2.D0 * DLM
        DOM2 = 2.D0 * DOM
C
        DH = - (17.2327D0 + 0.01737D0 * DT ) * sin(DOM)
     &       - ( 1.2729D0 + 0.00013D0 * DT ) * sin(DLS2)
     &       + 0.2088D0 * sin(DOM2) - 0.2037D0 * sin(DLM2)
     &       + ( 0.1261D0 - 0.00031D0 * DT) * sin(DMS)
     &       + 0.0675 * sin(DMM)
     &       - (0.0497D0 - 0.00012D0 * DT) * sin(DLS2+DMS)
     &       - 0.0342D0 * sin(DLM2-DOM)
     &       - 0.0261D0 * sin(DLM2+DMM)
     &       + 0.0214D0 * sin(DLS2-DMS)
     &       - 0.0149D0 * sin(DLS2-DLM2+DMM)
     &       + 0.0124D0 * sin(DLS2-DOM)
     &       + 0.0114D0 * sin(DLM2-DMM)
C
        DP = DH * DF
C
        DH =   ( 9.2100D0 + 0.00091D0 * DT ) * cos(DOM)
     &       + ( 0.5522D0 - 0.00029D0 * DT ) * cos(DLS2)
     &       - 0.0904D0 * cos(DOM2)
     &       + 0.0884D0 * cos(DLM2)
     &       + 0.0216D0 * cos(DLS2+DMS)
     &       + 0.0183D0 * cos(DLM2-DOM)
     &       + 0.0113D0 * cos(DLM2+DMM)
     &       - 0.0093D0 * cos(DLS2-DMS)
     &       - 0.0066D0 * cos(DLS2-DOM)
C
        DE = DH * DF
C
      endif
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
