! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine SIDJD1(DJD0,DSID,LMEAN,DJD,IERR)
C
CC    Calculates the time from the mean/apparent sid.time at Greenwich
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:37
C
CA  author : KOD        date: 02-OCT-1989 14:20
CU  update : KOD        date: 29-MAR-1990 16:43
CU  update : KOD        date: 29-MAR-1990 16:45
C
CT  status: tested
C
C   general description 
CG  Subroutine to calculate the Universal Time corresponding
CG  to a given mean/apparent sidereal time at Greenwich.
CG  This time will be selected to be nearest to the given date.
CG
CG  For consistency with the other subroutines the sidereal time
CG  is required in radians.
C
C   call_var.          type I/O description 
CP  DJD0                R8  I   Julian Date (nearest time)
CP  DSID                R8  I   sidereal time ( radians ! )
CP  LMEAN               L4  I   mean or app. sidereal time:
CP                                LMEAN = .true.  : mean sid. time
CP                                LMEAN = .false. : app. sid. time
CP  DJD                 R8    O Julian Date corresponding to DSID
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
CI  R_INCLUDE:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  JDSID1              SR      calculates the mean/app. sidereal time
CR  DRADRS              R8      'resets' radians
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'R_INCLUDE:ICGENL.INC'
      INCLUDE 'R_INCLUDE:IGENL.INC'
C
      REAL*8        DJD0, DSID, DJD, DSID0, DRADRS, DT, DJ0, DSD
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, ISTAT
      LOGICAL*4     LMEAN, LMN
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'SIDJD1'/
C
D     IF (HFLAG(RTNAME,'in: <input_parameters> ').NE.0) THEN
D       WRITE (OUTPT,101,IOSTAT=IOS) !<input_parameters>
D 101   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      IERR = 0
C
      if (DSID.lt.0.D0 .or. DSID.gt.D2PI) then
        IERR = 1
        call WRERR(RTNAME,IERR,'Error in the input parameter DSID')
        DSD = DSID
        DSD = DRADRS(DSD)
      endif
C
      if (IERR.lt.2) then
C
C       calculate the sidereal time at DJD0:
C
        DJ0 = DJD0
        LMN = LMEAN
        call JDSID1(DJ0,LMN,DSID0,ISTAT)
C
        if  (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        endif
C
      endif
C
      if (IERR.lt.2) then
C
C       calculate the difference in sidereal time:
C
        DT = DRADRS ( DSD - DSID0 ) / D2PI
C
        if (DT.gt.0.5D0) DT = DT - 1.D0
C
C       convert the difference into solar time:
C
        DT = DT / 1.002737908D0
C
        DJD = DJD0 + DT
C
      endif
C
C
D     IF (HFLAG(RTNAME,'out: <output_parameters> ').NE.0) THEN
D       WRITE (OUTPT,9901,IOSTAT=IOS) !<output_parameters>
D9901   FORMAT (  )
D       CALL WRFLAG(OUTPT)
D     ENDIF
C
      RETURN
      END
