! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine smbnd3(nband,iband,iseq,lcen,dalphm,dalphp,
     &                  ddeltm,ddeltp,ierr)
C
CC    Calculates the boundaries of a given small map
C
C************************ FFORM VERSION 1.2 ************  4-DEC-98 16:37
C
CA  author : KOD        date: 21-APR-1993
CU  update : GRU        date: 4-AUG-1994 15:05
CU  update : GRU        date: 4-AUG-1994 15:11
CU  update : GRU        date: 4-AUG-1994 15:12
C
CT  status: not tested
C
C   general description 
CG  Subroutine to calculate the boundaries of a given small map.
CG  Two polar caps, one band centered on the equator, with RA=0
CG  at the center (lcen=true) or border of small maps with iseq=1.
C
C   call_var.          type I/O description 
CP  NBAND               I4  I/? number of bands on the the sphere
CP                              including the equatorial band and
CP                              the polar caps
CP                              ( NBAND > 0  and  not even )
CP  IBAND               I4  I   actual number of the band     (1..nband)
CP  ISEQ                I4  I   sequence number along the band
CP  LCEN                L4  I   lcen=.true.  --> RA=0 at center
CP                              lcen=.false. --> RA=0 at border
CP  DALPHM              R8    O lower  Right Ascension     (deg, 0..360)
CP  DALPHP              R8    O higher Right Ascension     (deg, 0..360)
CP  DDELTM              R8    O lower Declination          (deg,-90..90)
CP  DDELTP              R8    O higher Declination         (deg,-90..90)
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  R_INCLUDE:ICGENL.INC        general parameter common block
C
C   routines_called    type     description 
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  SMCRT2              SR      
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      implicit none
      INCLUDE 'R_INCLUDE:ICGENL.INC'
c
      integer*4     maxbnd
c
      parameter     ( maxbnd = 100 )
c
      real*8        dalphm, dalphp, ddeltm, ddeltp, dh
      real*4        sbsm(maxbnd), rms
      integer*4     ierr, ios, istat, nband, ism, nband0, nbsm(maxbnd),
     &              nsm, i, iband, iseq
      logical       lcen, lfirst, lstart
      character*8   rtname
      save          nband0, nbsm
c
      data          rtname /'smbnd3'/
      data          lfirst / .true. /
C
      ierr = 0
c
c     check the input values:
c
      if (mod(nband,2).eq.0 .or. nband.gt.maxbnd .or.
     &    min(nband,iband,iseq).lt.1 .or. iband.gt.nband) then
        ierr = max(ierr,2)
        call wrerr(rtname,ierr,'Error in input values')
      endif
c
      if (ierr.lt.2) then
c
        if (lfirst) then
          lstart = .true.
        else
          lstart = (nband.ne.nband0)
        endif
c
        if (lstart) then
c
c         split the sphere into segments:
c
          call smcrt2(nband,nbsm,sbsm,nsm,rms,istat)
c
          if (istat.gt.1) then
            ierr = max(ierr,istat)
            call errstc(rtname,10)
          else
            nband0 = nband
            lfirst = .false.
          endif
c
        endif
      endif
c
      if (iseq.gt.nbsm(iband)) then
        ierr = max(ierr,2)
        write (outpt,105) iseq,iband,nbsm(iband)
105     format('iseq ',I5,' too large for band ',I5,' (max: ',I5,')')
        call wrerr(rtname,ierr,outpt)
      endif
c
      if (ierr.lt.2) then
c
c       calculate the declination boundaries:
c
        if (nband.eq.1) then
          ddeltm = -90.d0
          ddeltp =  90.d0
        else
          dh = 180.d0 / dble(nband-1)
          ddeltm = (0.5d0 * dble(nband) - dble(iband)) * dh
          ddeltp = dmin1( ddeltm + dh, 90.d0 )
          ddeltm = dmax1( ddeltm, -90.d0 )
        endif
c
c       calculate the right ascension boundaries:
c
        if (nband.eq.1 .or. ddeltm.eq.-90.d0 .or. ddeltp.eq.90.d0) then
          dalphm =   0.d0
          dalphp = 360.d0
        else
          dh = 360.d0 / dble(nbsm(iband))
          if (lcen) then
            dalphm = (dble(iseq)-1.5d0) * dh
          else
            dalphm = (dble(iseq)-1.0d0) * dh
          endif
          dalphp = dalphm + dh
          if (dalphm.lt.0.d0) dalphm = dalphm + 360.d0
        endif
c
      endif
c
      return
      end
