! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

      subroutine PHASE2(DT0,DPER,DT1,DPH1,DT,NPER,IERR)
C
CC    Calculates the time for a certain phase nearest to a given time
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:08
C
CA  author : KOD        date: 13-OCT-1989 14:48
CU  update : KOD        date: 29-MAR-1990 16:33
C
CT  status: tested
C
C   general description
CG  Subroutine to calculate the time nearest to a given time
CG  for which a certain phase occurs.
C
C   call_var.          type I/O description
CP  DT0                 R8  I   reference epoch
CP  DPER                R8  I   period             (same unit as DT0)
CP
CP  DT1                 R8  I   time of interest   (same unit as DT0)
CP  DPH1                R8  I   phase of interest    ( 0 .. 1)
CP
CP  DT                  R8    O corresponding time (same unit as DT0)
CP  NPER                I4    O number of extrapolated full periods
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description
CI  R$COMMON:CGENL.CMN          general parameter common block
C
C   routines_called    type     description
CR  PHASE1              SR      calculates the phase and number of per.
CR  RESVAL              SR      'resets' value
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description
CX
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c*
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
      CHARACTER*265 OUTPT                                ! R$INCLUDE:ICGENL.INC
      REAL*4        PI, RAD, DEG                         ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( PI  = 3.141592654 )                  ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( RAD = 0.0174532925 )                 ! R$INCLUDE:ICGENL.INC
      PARAMETER   ( DEG = 57.29577951 )                  ! R$INCLUDE:ICGENL.INC
c     ---------------------------------------------------! R$INCLUDE:ICGENL.INC
C
      REAL*8        DT0, DPER, DT1, DPH1, DT, DPH, DPHMIN, DPHMAX, DH,
     &              D0, D1, DP
      REAL*4        HFLAG
      INTEGER*4     IERR ,IOS, NPER, N, ISTAT
      CHARACTER*8   RTNAME
C
      DATA          RTNAME /'PHASE2'/
C
      IERR = 0
C
      if (DPH1.lt.0.D0 .or. DPH1.gt.1.D0) then
        IERR = max(IERR,1)
        call WRERR(RTNAME,IERR,'DPH1 not between 0 and 1')
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate phase for DT1:
C
        D0 = DT0
        DP = DPER
        D1 = DT1
        call PHASE1(D0,DP,D1,DPH,N,ISTAT)
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        endif
C
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate minimum difference in phase:
C
        DH = DPH1 - DPH
        DPHMIN = 0.D0
        DPHMAX = 1.D0
        call RESVAL(DPHMIN,DPHMAX,DH,DPH,ISTAT)
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,20)
        else
          if (DPH.ge.0.5D0) DPH = DPH - 1.D0
          DT = DT1 + DPH * DPER
        endif
C
      endif
C
C
      if (IERR.lt.2) then
C
C       calculate number of extrapolated full periods:
C
        call PHASE1(D0,DP,DT,DPH,N,ISTAT)
        NPER = N
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,30)
        endif
C
      endif
C
C
      RETURN
      END
